@@ -1,36 +1,36 @@
 /*------------------------------------------------------------------------------
-*
-* Copyright (c) 2011-2016, EURid. All rights reserved.
-* The YADIFA TM software product is provided under the BSD 3-clause license:
-* 
-* Redistribution and use in source and binary forms, with or without 
-* modification, are permitted provided that the following conditions
-* are met:
-*
-*        * Redistributions of source code must retain the above copyright 
-*          notice, this list of conditions and the following disclaimer.
-*        * Redistributions in binary form must reproduce the above copyright 
-*          notice, this list of conditions and the following disclaimer in the 
-*          documentation and/or other materials provided with the distribution.
-*        * Neither the name of EURid nor the names of its contributors may be 
-*          used to endorse or promote products derived from this software 
-*          without specific prior written permission.
-*
-* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
-* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
-* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
-* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
-* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
-* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
-* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
-* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
-* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
-* POSSIBILITY OF SUCH DAMAGE.
-*
-*------------------------------------------------------------------------------
-*
-*/
+ *
+ * Copyright (c) 2011-2016, EURid. All rights reserved.
+ * The YADIFA TM software product is provided under the BSD 3-clause license:
+ * 
+ * Redistribution and use in source and binary forms, with or without 
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ *        * Redistributions of source code must retain the above copyright 
+ *          notice, this list of conditions and the following disclaimer.
+ *        * Redistributions in binary form must reproduce the above copyright 
+ *          notice, this list of conditions and the following disclaimer in the 
+ *          documentation and/or other materials provided with the distribution.
+ *        * Neither the name of EURid nor the names of its contributors may be 
+ *          used to endorse or promote products derived from this software 
+ *          without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+ * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
+ * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
+ * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
+ * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
+ * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
+ * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+ * POSSIBILITY OF SUCH DAMAGE.
+ *
+ *------------------------------------------------------------------------------
+ *
+ */
 /** @defgroup streaming Streams
  *  @ingroup dnscore
  *  @brief
@@ -187,6 +187,94 @@ input_stream_read_u8(input_stream* stream, u8* output)
     return input_stream_read_fully(stream, output, 1);
 }
 
+union t32
+{
+    u8 bytes[4];
+    u32 value; 
+};
+
+ya_result
+input_stream_read_pu32(input_stream* is, u32* output)
+{
+    ya_result ret;
+    u32 value = 0;
+    union t32 buffer;
+    buffer.value = 0;
+    ya_result n = 0;
+    u8 s = 0;
+       
+    for(;;)
+    {
+#if WORDS_BIGENDIAN
+        if(FAIL(ret = input_stream_read(is, &buffer.bytes[3], 1)))
+        {
+            return ret;
+        }
+#else
+        if(FAIL(ret = input_stream_read(is, &buffer.bytes[0], 1)))
+        {
+            return ret;
+        }
+#endif
+        
+        value |= (buffer.value & 127) << s;
+        
+        ++n;
+        
+        if(buffer.value < 128)
+        {
+            *output = value;
+            return n;
+        }
+        
+        s += 7;
+    }
+}
+
+union t64
+{
+    u8 bytes[8];
+    u64 value; 
+};
+
+ya_result
+input_stream_read_pu64(input_stream* is, u64* output)
+{
+    ya_result ret;
+    u64 value = 0;
+    union t64 buffer;
+    buffer.value = 0;
+    ya_result n = 0;
+    u8 s = 0;
+       
+    for(;;)
+    {
+#if WORDS_BIGENDIAN
+        if(FAIL(ret = input_stream_read(is, &buffer.bytes[7], 1)))
+        {
+            return ret;
+        }
+#else
+        if(FAIL(ret = input_stream_read(is, &buffer.bytes[0], 1)))
+        {
+            return ret;
+        }
+#endif
+        
+        value |= (buffer.value & 127) << s;
+        
+        ++n;
+        
+        if(buffer.value < 128)
+        {
+            *output = value;
+            return n;
+        }
+        
+        s += 7;
+    }
+}
+
 ya_result
 input_stream_read_dnsname(input_stream* stream, u8* output_buffer)
 {
@@ -353,6 +441,7 @@ static const input_stream_vtbl void_input_stream_vtbl ={
 
 void input_stream_set_void(input_stream* stream)
 {
+    yassert(stream != NULL);
     stream->data = NULL;
     stream->vtbl = &void_input_stream_vtbl;
 }
