@@ -1,36 +1,36 @@
 /*------------------------------------------------------------------------------
-*
-* Copyright (c) 2011-2016, EURid. All rights reserved.
-* The YADIFA TM software product is provided under the BSD 3-clause license:
-* 
-* Redistribution and use in source and binary forms, with or without 
-* modification, are permitted provided that the following conditions
-* are met:
-*
-*        * Redistributions of source code must retain the above copyright 
-*          notice, this list of conditions and the following disclaimer.
-*        * Redistributions in binary form must reproduce the above copyright 
-*          notice, this list of conditions and the following disclaimer in the 
-*          documentation and/or other materials provided with the distribution.
-*        * Neither the name of EURid nor the names of its contributors may be 
-*          used to endorse or promote products derived from this software 
-*          without specific prior written permission.
-*
-* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
-* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
-* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
-* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
-* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
-* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
-* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
-* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
-* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
-* POSSIBILITY OF SUCH DAMAGE.
-*
-*------------------------------------------------------------------------------
-*
-*/
+ *
+ * Copyright (c) 2011-2016, EURid. All rights reserved.
+ * The YADIFA TM software product is provided under the BSD 3-clause license:
+ * 
+ * Redistribution and use in source and binary forms, with or without 
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ *        * Redistributions of source code must retain the above copyright 
+ *          notice, this list of conditions and the following disclaimer.
+ *        * Redistributions in binary form must reproduce the above copyright 
+ *          notice, this list of conditions and the following disclaimer in the 
+ *          documentation and/or other materials provided with the distribution.
+ *        * Neither the name of EURid nor the names of its contributors may be 
+ *          used to endorse or promote products derived from this software 
+ *          without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+ * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
+ * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
+ * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
+ * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
+ * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
+ * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+ * POSSIBILITY OF SUCH DAMAGE.
+ *
+ *------------------------------------------------------------------------------
+ *
+ */
 /** @defgroup dnscoretools Generic Tools
  *  @ingroup dnscore
  *  @brief
@@ -52,76 +52,10 @@
 #include "dnscore/parsing.h"
 
 #if !HAS_TIMEGM
-
-/// @note edf 20150326 -- timegm is not portable (Solaris) in the end, implementing one seemed the only choice
-
-/**
- * This implementation is based on the formula found in:
- * 
- * http://pubs.opengroup.org/onlinepubs/9699919799/basedefs/V1_chap04.html#tag_04_15
- * 4.15 Seconds Since the Epoch
- * 
- * It only cares on the Year, Month, Day, Hour, Minute and Second fields of struct tm (the one we are really caring about)
- * It's a constant-time implementation.
- * 
- * I don't see an obvious way to make it faster expect having the year values pre-calculated for the next 30+ years
- * (This would spare a few divs and a mult.)
- * 
- * If I improve it further, it may be a replacement on the timegm() dependency.
- * 
- */
-
-    // J   F   M    A    M    J    J    A    S    O    N    D
-    // 31  28  31   30   31   30   31   31   30   31   30   31
-
-#define MDAY_FIX(d_) ((d_)-1)
-
-static int timegm_mdays_norm[12] = {MDAY_FIX(0),MDAY_FIX( 31),MDAY_FIX( 59),MDAY_FIX( 90),MDAY_FIX( 120),MDAY_FIX( 151),MDAY_FIX( 181),MDAY_FIX( 212),MDAY_FIX( 243),MDAY_FIX( 273),MDAY_FIX( 304),MDAY_FIX( 334)}; // MDAY_FIX(365)
-
-static int timegm_mdays_leap[12] = {MDAY_FIX(0),MDAY_FIX( 31),MDAY_FIX( 60),MDAY_FIX( 91),MDAY_FIX( 121),MDAY_FIX( 152),MDAY_FIX( 182),MDAY_FIX( 213),MDAY_FIX( 244),MDAY_FIX( 274),MDAY_FIX( 305),MDAY_FIX( 335)}; // MDAY_FIX(366)
-
-static time_t timegm(struct tm *tv)
+static inline time_t timegm(struct tm *tv)
 {
-    time_t ret;
-    
-    if( (tv->tm_year < 0)                   ||
-        (((u32)tv->tm_mon) > 11)            ||
-        (((u32)tv->tm_mday - 1) > 31 - 1)   ||
-        (((u32)tv->tm_hour) > 60)           ||
-        (((u32)tv->tm_min) > 59)            ||
-        (((u32)tv->tm_sec) > 60) )
-    {
-        return -1;
-    }
-
-    int yyyy = (tv->tm_year + 1900);
-
-    int yday;
-    if(((yyyy & 3) == 0) && (((yyyy % 100) != 0) || ((yyyy % 400) == 0)))
-    {
-        yday = timegm_mdays_leap[tv->tm_mon];
-    }
-    else
-    {
-        yday = timegm_mdays_norm[tv->tm_mon];
-    }
-
-    yday += tv->tm_mday;
-
-    ret =   tv->tm_sec                       +
-            tv->tm_min               *    60 +
-            tv->tm_hour              *  3600 +
-            (
-                yday                     +
-                ((tv->tm_year-69)/4)     -
-                ((tv->tm_year-1)/100)    +
-                ((tv->tm_year+299)/400)
-            ) * 86400                        +
-            (tv->tm_year-70)         * 31536000;
-
-    return ret;
+    return timegm_internal(tv);
 }
-
 #endif
 
 /** \brief A string will be checked
@@ -177,16 +111,14 @@ parse_u32_check_range_len_base10(const char *src, u32 src_len, u32 *dst, u32 min
 {
     // 0......N
     // 67612321
-
-    --src_len;
     
-    if(src_len > 9)
+    if(src_len > 10)
     {
         return PARSEINT_ERROR; // out of range
     }
     
-    u32 base_multiplier = 10;
-    
+    --src_len;
+        
     u64 output_value = ((u64)src[src_len]) - '0';
     
     if((u64)output_value > 9)
@@ -194,13 +126,15 @@ parse_u32_check_range_len_base10(const char *src, u32 src_len, u32 *dst, u32 min
         return PARSEINT_ERROR;
     }
     
+    u32 base_multiplier = 10;
+    
     while(src_len > 0)
     {
         --src_len;
         
         u64 value = ((u64)src[src_len]) - '0';
         
-        if((u64)value > 9)
+        if(value > 9)
         {
             return PARSEINT_ERROR;
         }
@@ -285,6 +219,105 @@ parse_s32_check_range_len_base10(const char *src, u32 src_len, s32 *dst, s32 min
     return SUCCESS;
 }
 
+ya_result
+parse_u64_check_range_len_base10(const char *src, u32 src_len, u64 *dst, u64 min, u64 max)
+{
+    // 0......N
+    // 18446744073709551615
+    
+    if(src_len > 20)
+    {
+        return PARSEINT_ERROR; // out of range
+    }
+    
+    --src_len; // 19
+        
+    u64 output_value = ((u64)src[src_len]) - '0';
+    
+    if((u64)output_value > 9)
+    {
+        return PARSEINT_ERROR;
+    }
+    
+    if(src_len < 19) // if no risk of overflow
+    {
+        u64 base_multiplier = 10;
+
+        while(src_len > 0)
+        {
+            --src_len;
+
+            u64 value = ((u64)src[src_len]) - '0';
+
+            if(value > 9)
+            {
+                return PARSEINT_ERROR;
+            }
+
+            output_value *= base_multiplier;
+
+            output_value += value;
+
+            base_multiplier *= 10;
+        }
+    }
+    else // the only case with possible overflow at the last iteration of the loop
+    {
+        u64 base_multiplier = 10;
+        
+        while(src_len-- > 1)
+        {
+            u64 value = ((u64)src[src_len]) - '0';
+
+            if(value > 9)
+            {
+                return PARSEINT_ERROR;
+            }
+
+            output_value *= base_multiplier;
+
+            output_value += value;
+
+            base_multiplier *= 10;
+        }
+        
+        if(src_len == 0)
+        {
+            u64 max_div_10 = max / 10;
+            
+            if(output_value > max_div_10)   // check before multiplication there will be no 64 bits overflow
+            {                               // this only should be tested for the last iteration of the loop
+                return PARSEINT_ERROR;      // => the last pass should happen out of this loop
+            }
+
+            u64 value = ((u64)src[0]) - '0';
+
+            if(value > 9)
+            {
+                return PARSEINT_ERROR;
+            }
+
+            output_value *= base_multiplier;
+
+            if(output_value > max - value)  // check before addition there will be no 64 bits overflow
+            {
+                return PARSEINT_ERROR;
+            }
+
+            output_value += value;
+        }
+    }
+    
+    if((output_value < min) || (output_value > max)) // the second half of the test could probably get rid of, with a slight modification
+    {
+        return PARSEINT_ERROR;
+    }
+    
+    *dst = (u32)output_value;
+
+    return SUCCESS;
+}
+
 /** \brief Converts a string to an epoch
  *
  *  Converts a string to an epoch
@@ -296,7 +329,7 @@ parse_s32_check_range_len_base10(const char *src, u32 src_len, s32 *dst, s32 min
  *  @retval NOK, if no digits found, or number not in the range
  */
 ya_result
-parse_yyyymmddhhmmss_check_range_len(const char *src, u32 src_len, u32 *dst)
+parse_yyyymmddhhmmss_check_range_len(const char *src, u32 src_len, time_t *dst)
 {
     struct tm thetime;
     
@@ -306,7 +339,7 @@ parse_yyyymmddhhmmss_check_range_len(const char *src, u32 src_len, u32 *dst)
     }
 
 #ifdef DEBUG
-    memset(&thetime, 0xff, sizeof (thetime));
+    memset(&thetime, 0xff, sizeof(thetime));
 #endif
     
     u32 tmp_u32;
@@ -368,7 +401,7 @@ parse_yyyymmddhhmmss_check_range_len(const char *src, u32 src_len, u32 *dst)
 }
 
 ya_result
-parse_yyyymmddhhmmss_check_range(const char *src, u32 *dst)
+parse_yyyymmddhhmmss_check_range(const char *src, time_t *dst)
 {
     ya_result return_code;
     
