@@ -1,36 +1,36 @@
 /*------------------------------------------------------------------------------
-*
-* Copyright (c) 2011-2016, EURid. All rights reserved.
-* The YADIFA TM software product is provided under the BSD 3-clause license:
-* 
-* Redistribution and use in source and binary forms, with or without 
-* modification, are permitted provided that the following conditions
-* are met:
-*
-*        * Redistributions of source code must retain the above copyright 
-*          notice, this list of conditions and the following disclaimer.
-*        * Redistributions in binary form must reproduce the above copyright 
-*          notice, this list of conditions and the following disclaimer in the 
-*          documentation and/or other materials provided with the distribution.
-*        * Neither the name of EURid nor the names of its contributors may be 
-*          used to endorse or promote products derived from this software 
-*          without specific prior written permission.
-*
-* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
-* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
-* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
-* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
-* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
-* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
-* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
-* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
-* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
-* POSSIBILITY OF SUCH DAMAGE.
-*
-*------------------------------------------------------------------------------
-*
-*/
+ *
+ * Copyright (c) 2011-2016, EURid. All rights reserved.
+ * The YADIFA TM software product is provided under the BSD 3-clause license:
+ * 
+ * Redistribution and use in source and binary forms, with or without 
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ *        * Redistributions of source code must retain the above copyright 
+ *          notice, this list of conditions and the following disclaimer.
+ *        * Redistributions in binary form must reproduce the above copyright 
+ *          notice, this list of conditions and the following disclaimer in the 
+ *          documentation and/or other materials provided with the distribution.
+ *        * Neither the name of EURid nor the names of its contributors may be 
+ *          used to endorse or promote products derived from this software 
+ *          without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+ * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
+ * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
+ * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
+ * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
+ * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
+ * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+ * POSSIBILITY OF SUCH DAMAGE.
+ *
+ *------------------------------------------------------------------------------
+ *
+ */
 /** @defgroup dnsdbzone Zone related functions
  *  @ingroup dnsdb
  *  @brief Functions used to manipulate a zone
@@ -46,13 +46,19 @@
 #include <dnscore/input_stream.h>
 #include <dnscore/output_stream.h>
 
+#if HAS_DNSSEC_SUPPORT
+#include <dnscore/dnskey.h>
+#endif
+
 #include <dnsdb/zdb_types.h>
 #include <dnsdb/dnsrdata.h>
 
 #include <dnsdb/zdb-zone-lock.h>
 #include <dnsdb/zdb-zone-arc.h>
 
-#include "nsec3_types.h"
+#if ZDB_HAS_NSEC3_SUPPORT
+#include <dnsdb/nsec3_types.h>
+#endif
 
 #ifdef	__cplusplus
 extern "C"
@@ -108,19 +114,34 @@ void zdb_zone_record_add(zdb_zone* zone, dnslabel_vector_reference labels, s32 l
 /**
  * @brief Removes a record from a zone
  *
- * Removes a record from a zone
+ * Removes a record from a zone and frees its memory.
+ * 
+ * In the current version (20160513) if the record itself should not be used as
+ * the parameter is still used after removal for feedback to the journal.
  *
  * @param[in] zone the zone
  * @param[in] labels the stack of labels of the dns name
  * @param[in] labels_top the index of the top of the stack (the level)
  * @param[in] type the type of the record
  * @param[in] ttlrdata the ttl and rdata of the record.  NOTE: the caller stays the owner
- *
- * @return an error code
  */
 
+ya_result zdb_zone_record_delete(zdb_zone *zone, dnslabel_vector_reference labels, s32 labels_top, u16 type, zdb_packed_ttlrdata* packed_ttlrdata);
 
-ya_result zdb_zone_record_delete(zdb_zone* zone, dnslabel_vector_reference labels, s32 labels_top, u16 type, zdb_packed_ttlrdata* ttlrdata); /* class is implicit */
+/**
+ * @brief Removes a record from a zone
+ *
+ * Copies the record then removes it from the zone and frees its memory.
+ * This allows removing safely a record using itself as a parameter.
+ *
+ * @param[in] zone the zone
+ * @param[in] labels the stack of labels of the dns name
+ * @param[in] labels_top the index of the top of the stack (the level)
+ * @param[in] type the type of the record
+ * @param[in] ttlrdata the ttl and rdata of the record.  NOTE: the caller stays the owner
+ */
+
+ya_result zdb_zone_record_delete_self(zdb_zone *zone, dnslabel_vector_reference labels, s32 labels_top, u16 type, zdb_packed_ttlrdata* packed_ttlrdata);
 
 /**
  * @brief Search for a record in a zone
@@ -259,12 +280,11 @@ ya_result zdb_zone_update_ixfr(zdb *db, input_stream *is);
 
 ya_result zdb_zone_store_axfr(zdb_zone *zone, output_stream* os);
 
-const zdb_packed_ttlrdata*
-zdb_zone_get_dnskey_rrset(zdb_zone *zone);
+const zdb_packed_ttlrdata *zdb_zone_get_dnskey_rrset(zdb_zone *zone);
 
 static inline bool zdb_zone_is_nsec3(const zdb_zone* zone)
 {
-#if ZDB_HAS_NSEC3_SUPPORT != 0
+#if ZDB_HAS_NSEC3_SUPPORT
     return (zone->apex->flags & ZDB_RR_LABEL_NSEC3) != 0;
 #else
     return FALSE;
@@ -273,31 +293,37 @@ static inline bool zdb_zone_is_nsec3(const zdb_zone* zone)
 
 static inline bool zdb_zone_is_nsec3_optout(const zdb_zone* zone)
 {
+#if ZDB_HAS_NSEC3_SUPPORT
     return ((zone->apex->flags & ZDB_RR_LABEL_NSEC3_OPTOUT) != 0);
+#else
+    return FALSE;
+#endif
 }
 
 static inline bool zdb_zone_is_nsec3_optin(const zdb_zone* zone)
 {
+#if ZDB_HAS_NSEC3_SUPPORT
     return ((zone->apex->flags & ZDB_RR_LABEL_NSEC3_OPTOUT) == 0);
+#else
+    return FALSE;
+#endif
 }
 
+#if ZDB_HAS_NSEC3_SUPPORT
 static inline nsec3_zone* zdb_zone_get_nsec3chain(const zdb_zone* zone, s8 idx)
 {
-#if ZDB_HAS_NSEC3_SUPPORT != 0
-    nsec3_zone *n3 = zone->nsec.nsec3;;
+    nsec3_zone *n3 = zone->nsec.nsec3;
     while(n3 != NULL && --idx >= 0)
     {
         n3 = n3->next;
     }
     return n3;
-#else
-    return NULL;
-#endif
 }
+#endif
 
 static inline bool zdb_zone_is_nsec(const zdb_zone *zone)
 {
-#if ZDB_HAS_NSEC_SUPPORT != 0
+#if ZDB_HAS_NSEC_SUPPORT
     return (zone->apex->flags & ZDB_RR_LABEL_NSEC) != 0;
 #else
     return FALSE;
@@ -306,13 +332,27 @@ static inline bool zdb_zone_is_nsec(const zdb_zone *zone)
 
 static inline bool zdb_zone_is_dnssec(const zdb_zone *zone)
 {
-#if ZDB_HAS_DNSSEC_SUPPORT != 0
+#if ZDB_HAS_DNSSEC_SUPPORT
     return (zone->apex->flags & (ZDB_RR_LABEL_NSEC | ZDB_RR_LABEL_NSEC3)) != 0;
 #else
     return FALSE;
 #endif    
 }
 
+#if OBSOLETE
+/**
+ * Exchange the current zone with a dummy invalid one.
+ * Do nothing if the zone in place is already invalid.
+ * Returns the previous zone.
+ * 
+ * @param db
+ * @param origin
+ * @return 
+ */
+
+zdb_zone *zdb_zone_xchg_with_invalid(zdb *db, const u8 *origin, u16 or_flags);
+
+#endif
 
 bool zdb_zone_isinvalid(zdb_zone *zone);
 
@@ -321,6 +361,53 @@ static inline bool zdb_zone_isvalid(zdb_zone *zone)
     return !zdb_zone_isinvalid(zone);
 }
 
+#if HAS_DNSSEC_SUPPORT
+
+/**
+ * Adds a DNSKEY record in a zone from the dnssec_key object.
+ * 
+ * @param key
+ * @return TRUE iff the record has been added
+ */
+
+
+bool zdb_zone_add_dnskey_from_key(zdb_zone *zone, const dnssec_key *key);
+
+/**
+ * Removes a DNSKEY record in a zone from the dnssec_key object.
+ * 
+ * @param key
+ * @return TRUE iff the record has been found and removed
+ */
+
+
+bool zdb_zone_remove_dnskey_from_key(zdb_zone *zone, const dnssec_key *key);
+
+/**
+ * 
+ * Returns TRUE iff the key is present as a record in the zone
+ * 
+ * @param zone
+ * @param key
+ * @return 
+ */
+
+bool zdb_zone_contains_dnskey_record_for_key(zdb_zone *zone, const dnssec_key *key);
+
+/**
+ * Returns TRUE iff there is at least one RRSIG record with the tag and algorithm of the key
+ * 
+ * @param zone
+ * @param key
+ * @return 
+ */
+
+bool zdb_zone_apex_contains_rrsig_record_by_key(zdb_zone *zone, const dnssec_key *key);
+
+void zdb_zone_update_keystore_keys_from_zone(zdb_zone *zone);
+
+#endif
+
 #ifdef DEBUG
 
 /**
