@@ -1,36 +1,36 @@
 /*------------------------------------------------------------------------------
-*
-* Copyright (c) 2011-2016, EURid. All rights reserved.
-* The YADIFA TM software product is provided under the BSD 3-clause license:
-* 
-* Redistribution and use in source and binary forms, with or without 
-* modification, are permitted provided that the following conditions
-* are met:
-*
-*        * Redistributions of source code must retain the above copyright 
-*          notice, this list of conditions and the following disclaimer.
-*        * Redistributions in binary form must reproduce the above copyright 
-*          notice, this list of conditions and the following disclaimer in the 
-*          documentation and/or other materials provided with the distribution.
-*        * Neither the name of EURid nor the names of its contributors may be 
-*          used to endorse or promote products derived from this software 
-*          without specific prior written permission.
-*
-* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
-* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
-* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
-* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
-* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
-* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
-* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
-* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
-* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
-* POSSIBILITY OF SUCH DAMAGE.
-*
-*------------------------------------------------------------------------------
-*
-*/
+ *
+ * Copyright (c) 2011-2016, EURid. All rights reserved.
+ * The YADIFA TM software product is provided under the BSD 3-clause license:
+ * 
+ * Redistribution and use in source and binary forms, with or without 
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ *        * Redistributions of source code must retain the above copyright 
+ *          notice, this list of conditions and the following disclaimer.
+ *        * Redistributions in binary form must reproduce the above copyright 
+ *          notice, this list of conditions and the following disclaimer in the 
+ *          documentation and/or other materials provided with the distribution.
+ *        * Neither the name of EURid nor the names of its contributors may be 
+ *          used to endorse or promote products derived from this software 
+ *          without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+ * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
+ * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
+ * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
+ * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
+ * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
+ * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+ * POSSIBILITY OF SUCH DAMAGE.
+ *
+ *------------------------------------------------------------------------------
+ *
+ */
 /** @defgroup nsec3 NSEC3 functions
  *  @ingroup dnsdbdnssec
  *  @brief
@@ -242,6 +242,82 @@ AVL_PREFIXED(avl_find_interval_start)(AVL_TREE_TYPE* root, AVL_REFERENCE_TYPE ob
     return lower_bound;
 }
 
+AVL_NODE_TYPE*
+AVL_PREFIXED(avl_find_prev_mod)(AVL_TREE_TYPE* root, AVL_REFERENCE_TYPE obj_hash)
+{
+    AVL_NODE_TYPE* node = *root;
+    AVL_NODE_TYPE* lower_bound = NULL;
+    AVL_REFERENCE_TYPE h;
+    
+    yassert(node != NULL);
+
+    /* This is one of the parts I could try to optimize
+     * I've checked the assembly, and it sucks ...
+     */
+
+    /* Both the double-test while/ternary and the current one
+     * are producing the same assembly code.
+     */
+    
+    /**
+     * Get a key that 
+     */
+
+    while(node != NULL)
+    {
+        h = AVL_REFERENCE(node);
+
+        /*
+         * [0] is the length of the obj_hash
+         *
+         * The obj_hashs starts at [1]
+         *
+         */
+
+#ifdef DEBUG
+        if(h[0] != obj_hash[0])
+        {
+            DIE_MSG("NSEC3 corrupted NSEC3 node");
+        }
+#endif
+
+        int cmp = memcmp(&obj_hash[1], &h[1], h[0]);
+
+        /* equals */
+        if(cmp == 0)
+        {
+            // want the prev mod
+            
+            return nsec3_avl_node_mod_prev(node);
+        }
+
+        /* bigger */
+        if(cmp > 0)
+        {
+            lower_bound = node;
+            node = AVL_CHILD(node, DIR_RIGHT);            
+        }
+        else
+        {
+            node = AVL_CHILD(node, DIR_LEFT);
+        }
+    }
+    
+    if(lower_bound == NULL)
+    {
+        lower_bound = *root;
+        
+        yassert(lower_bound != NULL);
+        
+        while((node = AVL_CHILD(lower_bound, DIR_RIGHT)) != NULL)
+        {
+            lower_bound = node;
+        }
+    }
+    
+    return lower_bound;
+}
+
 /** @} */
 
 /*----------------------------------------------------------------------------*/
