@@ -1,36 +1,36 @@
 /*------------------------------------------------------------------------------
-*
-* Copyright (c) 2011-2016, EURid. All rights reserved.
-* The YADIFA TM software product is provided under the BSD 3-clause license:
-* 
-* Redistribution and use in source and binary forms, with or without 
-* modification, are permitted provided that the following conditions
-* are met:
-*
-*        * Redistributions of source code must retain the above copyright 
-*          notice, this list of conditions and the following disclaimer.
-*        * Redistributions in binary form must reproduce the above copyright 
-*          notice, this list of conditions and the following disclaimer in the 
-*          documentation and/or other materials provided with the distribution.
-*        * Neither the name of EURid nor the names of its contributors may be 
-*          used to endorse or promote products derived from this software 
-*          without specific prior written permission.
-*
-* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
-* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
-* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
-* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
-* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
-* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
-* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
-* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
-* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
-* POSSIBILITY OF SUCH DAMAGE.
-*
-*------------------------------------------------------------------------------
-*
-*/
+ *
+ * Copyright (c) 2011-2016, EURid. All rights reserved.
+ * The YADIFA TM software product is provided under the BSD 3-clause license:
+ * 
+ * Redistribution and use in source and binary forms, with or without 
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ *        * Redistributions of source code must retain the above copyright 
+ *          notice, this list of conditions and the following disclaimer.
+ *        * Redistributions in binary form must reproduce the above copyright 
+ *          notice, this list of conditions and the following disclaimer in the 
+ *          documentation and/or other materials provided with the distribution.
+ *        * Neither the name of EURid nor the names of its contributors may be 
+ *          used to endorse or promote products derived from this software 
+ *          without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+ * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
+ * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
+ * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
+ * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
+ * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
+ * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+ * POSSIBILITY OF SUCH DAMAGE.
+ *
+ *------------------------------------------------------------------------------
+ *
+ */
 /** @defgroup records Internal functions for the database: resource records.
  *  @ingroup dnsdb
  *  @brief Internal functions for the database: resource records.
@@ -58,6 +58,10 @@
 
 #include "dnsdb/btree.h"
 
+#if HAS_NSEC3_SUPPORT
+#include "dnsdb/nsec3_types.h"
+#endif
+
 #define TTLRDATA_TAG 0x41544144524c5454
 #define ZDBRDATA_TAG 0x415441445242445a
 
@@ -74,6 +78,13 @@ zdb_record_free(zdb_packed_ttlrdata* record)
     /** MEMORY CANNOT BE TRASHED WHEN USING ZALLOC BECAUSE rdata_size IS USED TO FREE THE MEMORY ! */
     /** DO NOT DO THIS : memset(record,0x8f,sizeof(zdb_packed_ttlrdata)+record->rdata_size-1); */
 
+#ifdef DEBUG
+    //yassert((u32)(intptr)(record->next) != 0xfefefefeUL);
+    u16 tmp = record->rdata_size;
+    memset(record, 0xfe, sizeof(zdb_packed_ttlrdata) + tmp - 1);
+    record->rdata_size = tmp;
+#endif
+    
     ZDB_RECORD_ZFREE(record);
 }
 
@@ -98,6 +109,25 @@ zdb_record_insert(zdb_rr_collection* collection, u16 type, zdb_packed_ttlrdata*
 {
     zdb_packed_ttlrdata** record_sll = (zdb_packed_ttlrdata**)btree_insert(collection, type);
 
+#ifdef DEBUG
+    switch(type)
+    {
+        case TYPE_A:
+            if(record->rdata_size != 4) abort();
+            break;
+        case TYPE_AAAA:
+            if(record->rdata_size != 16) abort();
+            break;
+#if ZDB_HAS_NSEC3_SUPPORT
+        case TYPE_NSEC3PARAM:
+            if(record->rdata_size != NSEC3PARAM_RDATA_SIZE_FROM_RDATA(record->rdata_start)) abort();
+            break;
+#endif
+        default:
+            break;
+    }
+#endif
+    
     record->next = *record_sll;
     *record_sll = record;
 }
@@ -138,6 +168,25 @@ zdb_record_insert_checked(zdb_rr_collection* collection, u16 type, zdb_packed_tt
             {
                 if(memcmp(next->rdata_start, record->rdata_start, record->rdata_size) == 0) /* dup */
                 {
+#ifdef DEBUG
+                    switch(type)
+                    {
+                        case TYPE_A:
+                            if(record->rdata_size != 4) abort();
+                            break;
+                        case TYPE_AAAA:
+                            if(record->rdata_size != 16) abort();
+                            break;
+#if ZDB_HAS_NSEC3_SUPPORT
+                        case TYPE_NSEC3PARAM:
+                            if(record->rdata_size != NSEC3PARAM_RDATA_SIZE_FROM_RDATA(record->rdata_start)) abort();
+                            break;
+#endif
+                        default:
+                            break;
+                    }
+#endif
+                    
                     next = next->next;
                     
                     while(next != NULL)
@@ -167,6 +216,81 @@ zdb_record_insert_checked(zdb_rr_collection* collection, u16 type, zdb_packed_tt
     return TRUE;
 }
 
+/** @brief Inserts a resource record into the resource collection, checks for dups
+ *
+ *  Do not assume anything.
+ *  Inserts a ttl-rdata record into the rtl-rdata collection
+ *  The caller loses the property of the record.
+ *  If the record is a dup, it is destroyed.
+ *  TTL value is not propagated through the resource record set
+ *
+ *  @param[in]  collection the collection
+ *  @param[in]  class_ the class of the resource record
+ *  @param[in]  type the type of the resource record
+ *  @param[in]  ttl the ttl of the resource record
+ *  @param[in]  rdata_size the size of the rdata of the resource record
+ *  @param[in]  rdata a pointer to the rdata of the resource record
+ *
+ *  @return TRUE in case of success.
+ */
+
+bool
+zdb_record_insert_checked_keep_ttl(zdb_rr_collection* collection, u16 type, zdb_packed_ttlrdata* record)
+{
+    zdb_packed_ttlrdata** record_sll = (zdb_packed_ttlrdata**)btree_insert(collection, type);
+    
+    if(type != TYPE_CNAME)
+    {
+        zdb_packed_ttlrdata* next = *record_sll;
+        
+        while(next != NULL)
+        {
+            if(next->rdata_size == record->rdata_size)
+            {
+                if(memcmp(next->rdata_start, record->rdata_start, record->rdata_size) == 0) /* dup */
+                {
+#ifdef DEBUG
+                    switch(type)
+                    {
+                        case TYPE_A:
+                            if(record->rdata_size != 4) abort();
+                            break;
+                        case TYPE_AAAA:
+                            if(record->rdata_size != 16) abort();
+                            break;
+#if ZDB_HAS_NSEC3_SUPPORT
+                        case TYPE_NSEC3PARAM:
+                            if(record->rdata_size != NSEC3PARAM_RDATA_SIZE_FROM_RDATA(record->rdata_start)) abort();
+                            break;
+#endif
+                        default:
+                            break;
+                    }
+#endif
+                    
+                    next = next->next;
+                    
+                    return FALSE;
+                }
+            }
+
+            next = next->next;
+        }
+
+        record->next = *record_sll;
+        *record_sll = record;
+    }
+    else
+    {
+        ZDB_RECORD_SAFE_ZFREE(*record_sll);
+        record->next = NULL;
+        *record_sll = record;
+    }
+
+    return TRUE;
+}
+
+
 /** @brief Finds and return all the a resource record matching the class and type
  *
  *  Finds and returns all the a resource record matching the class and type
@@ -341,7 +465,7 @@ zdb_record_equals(const zdb_packed_ttlrdata *a, const zdb_packed_ttlrdata *b)
  */
 
 ya_result
-zdb_record_delete_exact(zdb_rr_collection* collection, u16 type, const zdb_ttlrdata* ttlrdata)
+zdb_record_delete_exact(zdb_rr_collection* collection, u16 type, const zdb_ttlrdata *ttlrdata)
 {
     yassert((collection != NULL) && (type != TYPE_ANY));
 
@@ -376,6 +500,8 @@ zdb_record_delete_exact(zdb_rr_collection* collection, u16 type, const zdb_ttlrd
 
                         ret = SUCCESS_LAST_RECORD;                  /* There is still at least one record of this type available */
                     }
+                    
+                    yassert(record_list->rdata_start != ttlrdata->rdata_pointer);
 
                     zdb_record_free(record_list);
 
@@ -384,6 +510,8 @@ zdb_record_delete_exact(zdb_rr_collection* collection, u16 type, const zdb_ttlrd
 
                 prev->next = record_list->next;
 
+                yassert(record_list->rdata_start != ttlrdata->rdata_pointer);
+                
                 zdb_record_free(record_list);
 
                 return SUCCESS_STILL_RECORDS; /* There is still at least one record of this type available */
@@ -399,6 +527,37 @@ zdb_record_delete_exact(zdb_rr_collection* collection, u16 type, const zdb_ttlrd
     return ZDB_ERROR_KEY_NOTFOUND;
 }
 
+ya_result
+zdb_record_delete_self_exact(zdb_rr_collection* collection, u16 type, const zdb_ttlrdata *ttlrdata_)
+{
+    u8 *tmp;
+    ya_result ret;
+    u8 tmp_[512];
+    zdb_ttlrdata ttlrdata;
+    ttlrdata.next = NULL;
+    ttlrdata.rdata_size = ttlrdata_->rdata_size;
+    if(ttlrdata.rdata_size <= sizeof(tmp_))
+    {
+        tmp = &tmp_[0];
+    }
+    else   
+    {
+        MALLOC_OR_DIE(u8*,tmp,ttlrdata.rdata_size,GENERIC_TAG);
+    }
+    memcpy(tmp, ttlrdata_->rdata_pointer, ttlrdata.rdata_size);
+    ttlrdata.rdata_pointer = tmp;
+    ttlrdata.ttl = ttlrdata_->ttl;
+
+    ret = zdb_record_delete_exact(collection, type, &ttlrdata); // safe
+    
+    if(ttlrdata.rdata_size > sizeof(tmp_))
+    {
+        free(tmp);
+    }
+    
+    return ret;
+}
+
 static void
 zdb_record_destroy_callback(void* record_list_)
 {
@@ -502,14 +661,36 @@ zdb_record_getsoa(const zdb_packed_ttlrdata* soa, soa_rdata* soa_out)
  * @brief Allocated and duplicates the content of the source
  */
 
-zdb_ttlrdata* zdb_ttlrdata_clone(const zdb_ttlrdata* source)
+zdb_ttlrdata*
+zdb_ttlrdata_clone(const zdb_ttlrdata* source)
 {
     zdb_ttlrdata *rec;
-    MALLOC_OR_DIE(zdb_ttlrdata*, rec, sizeof(zdb_ttlrdata), TTLRDATA_TAG); /// @todo edf 20141001 -- maybe merge the two allocs into one
+    int size = ((sizeof(zdb_ttlrdata) + 7) & ~7) + source->rdata_size;
+    ZALLOC_ARRAY_OR_DIE(zdb_ttlrdata*, rec, size, TTLRDATA_TAG);
     rec->next = NULL;
     rec->ttl = source->ttl;
     rec->rdata_size = source->rdata_size;
-    MALLOC_OR_DIE(u8*, rec->rdata_pointer, rec->rdata_size, ZDBRDATA_TAG);
+    rec->rdata_pointer = &((u8*)rec)[(sizeof(zdb_ttlrdata) + 7) & ~7];
+    memcpy(rec->rdata_pointer, source->rdata_pointer, rec->rdata_size);
+    
+    return rec;
+}
+
+/**
+ * @brief Allocated and duplicates the first bytes of content of the source
+ * This is mostly used to clone an NSEC3 record into an NSEC3PARAM
+ */
+
+zdb_ttlrdata*
+zdb_ttlrdata_clone_resized(const zdb_ttlrdata* source, u32 rdata_newsize)
+{
+    zdb_ttlrdata *rec;
+    int size = ((sizeof(zdb_ttlrdata) + 7) & ~7) + rdata_newsize;
+    ZALLOC_ARRAY_OR_DIE(zdb_ttlrdata*, rec, size, TTLRDATA_TAG);
+    rec->next = NULL;
+    rec->ttl = source->ttl;
+    rec->rdata_size = rdata_newsize;
+    rec->rdata_pointer = &((u8*)rec)[(sizeof(zdb_ttlrdata) + 7) & ~7];
     memcpy(rec->rdata_pointer, source->rdata_pointer, rec->rdata_size);
     
     return rec;
@@ -520,8 +701,8 @@ zdb_ttlrdata* zdb_ttlrdata_clone(const zdb_ttlrdata* source)
  */
 void zdb_ttlrdata_delete(zdb_ttlrdata* record)
 {
-    free(record->rdata_pointer);
-    free(record);
+    int size = ((sizeof(zdb_ttlrdata) + 7) & ~7) + record->rdata_size;
+    ZFREE_ARRAY(record, size);
 }
 
 #ifdef DEBUG
