@@ -1,36 +1,36 @@
 /*------------------------------------------------------------------------------
-*
-* Copyright (c) 2011-2016, EURid. All rights reserved.
-* The YADIFA TM software product is provided under the BSD 3-clause license:
-* 
-* Redistribution and use in source and binary forms, with or without 
-* modification, are permitted provided that the following conditions
-* are met:
-*
-*        * Redistributions of source code must retain the above copyright 
-*          notice, this list of conditions and the following disclaimer.
-*        * Redistributions in binary form must reproduce the above copyright 
-*          notice, this list of conditions and the following disclaimer in the 
-*          documentation and/or other materials provided with the distribution.
-*        * Neither the name of EURid nor the names of its contributors may be 
-*          used to endorse or promote products derived from this software 
-*          without specific prior written permission.
-*
-* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
-* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
-* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
-* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
-* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
-* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
-* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
-* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
-* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
-* POSSIBILITY OF SUCH DAMAGE.
-*
-*------------------------------------------------------------------------------
-*
-*/
+ *
+ * Copyright (c) 2011-2016, EURid. All rights reserved.
+ * The YADIFA TM software product is provided under the BSD 3-clause license:
+ * 
+ * Redistribution and use in source and binary forms, with or without 
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ *        * Redistributions of source code must retain the above copyright 
+ *          notice, this list of conditions and the following disclaimer.
+ *        * Redistributions in binary form must reproduce the above copyright 
+ *          notice, this list of conditions and the following disclaimer in the 
+ *          documentation and/or other materials provided with the distribution.
+ *        * Neither the name of EURid nor the names of its contributors may be 
+ *          used to endorse or promote products derived from this software 
+ *          without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+ * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
+ * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
+ * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
+ * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
+ * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
+ * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+ * POSSIBILITY OF SUCH DAMAGE.
+ *
+ *------------------------------------------------------------------------------
+ *
+ */
 /** @defgroup dnsdbzone Zone related functions
  *  @ingroup dnsdb
  *  @brief Functions used to manipulate a zone
@@ -55,7 +55,7 @@
 #include <dnscore/threaded_dll_cw.h>
 
 #include "dnsdb/dnsdb-config.h"
-
+#include "dnsdb/zdb_icmtl.h"
 #include "dnsdb/zdb.h"
 
 #include "dnsdb/zdb_zone.h"
@@ -72,12 +72,15 @@
 
 #include "dnsdb/journal.h"
 
+#if HAS_DNSSEC_SUPPORT
+#include "dnsdb/rrsig.h"
 #if ZDB_HAS_NSEC_SUPPORT != 0
 #include "dnsdb/nsec.h"
 #endif
 #if ZDB_HAS_NSEC3_SUPPORT != 0
 #include "dnsdb/nsec3.h"
 #endif
+#endif
 
 #ifdef DEBUG
 #define ZONE_MUTEX_LOG 0    // set this to 0 to disable in DEBUG
@@ -97,6 +100,9 @@ ptr_set g_zone_instanciated_set = PTR_SET_PTR_EMPTY;
  * @brief Adds a record to a zone
  *
  * Adds a record to a zone.
+ * 
+ * @note Expects the full fqdn in the labels parameter. "." has a labels_top at -1
+ * 
  *
  * @param[in] zone the zone
  * @param[in] labels the stack of labels of the dns name
@@ -106,12 +112,12 @@ ptr_set g_zone_instanciated_set = PTR_SET_PTR_EMPTY;
  */
 
 void
-zdb_zone_record_add(zdb_zone *zone, dnslabel_vector_reference labels, s32 labels_top, u16 type, zdb_packed_ttlrdata* ttlrdata)
+zdb_zone_record_add(zdb_zone *zone, dnslabel_vector_reference labels, s32 labels_top, u16 type, zdb_packed_ttlrdata *ttlrdata)
 {
-    zdb_rr_label* rr_label = zdb_rr_label_add(zone, labels, labels_top);
+    zdb_rr_label *rr_label = zdb_rr_label_add(zone, labels, labels_top - zone->origin_vector.size - 1); // flow verified
     /* This record will be put as it is in the DB */
 
-#if ZDB_HAS_NSEC_SUPPORT != 0
+#if ZDB_HAS_NSEC_SUPPORT
     /*
      * At this point I could add empty nsec3 records, or schedule the nsec3 signature
      */
@@ -133,6 +139,33 @@ zdb_zone_record_add(zdb_zone *zone, dnslabel_vector_reference labels, s32 labels
             break;
         }
         case TYPE_RRSIG:
+        {
+            if(!zdb_record_insert_checked_keep_ttl(&rr_label->resource_record_set, type, ttlrdata)) /* FB done */
+            {
+                ZDB_RECORD_ZFREE(ttlrdata);
+                return;
+            }
+
+            rr_label->flags |= flag_mask;
+
+#if ZDB_CHANGE_FEEDBACK_SUPPORT != 0
+
+            /*
+             * Update ICMTL.
+             *
+             * NOTE: the zdb_rr_label set of functions are zdb_listener-aware but the zdb_record ones are not.
+             * That's why this one needs a call to the listener.
+             *
+             */
+
+            zdb_ttlrdata unpacked_ttlrdata;
+            unpacked_ttlrdata.rdata_pointer = &ttlrdata->rdata_start[0];
+            unpacked_ttlrdata.rdata_size = ttlrdata->rdata_size;
+            unpacked_ttlrdata.ttl = ttlrdata->ttl;
+            zdb_listener_notify_add_record(zone, labels, labels_top, type, &unpacked_ttlrdata);
+#endif
+            return;
+        }
         case TYPE_NSEC:
             break;
         case TYPE_NS:
@@ -206,7 +239,10 @@ zdb_zone_record_add(zdb_zone *zone, dnslabel_vector_reference labels, s32 labels
 /**
  * @brief Removes a record from a zone
  *
- * Removes a record from a zone
+ * Removes a record from a zone and frees its memory.
+ * 
+ * In the current version (20160513) if the record itself should not be used as
+ * the parameter is still used after removal for feedback to the journal.
  *
  * @param[in] zone the zone
  * @param[in] labels the stack of labels of the dns name
@@ -226,7 +262,51 @@ zdb_zone_record_delete(zdb_zone *zone, dnslabel_vector_reference labels, s32 lab
     ttlrdata.rdata_pointer = ZDB_PACKEDRECORD_PTR_RDATAPTR(packed_ttlrdata);
     ttlrdata.ttl = packed_ttlrdata->ttl;
 
-    return zdb_rr_label_delete_record_exact(zone, labels, labels_top, type, &ttlrdata);
+    return zdb_rr_label_delete_record_exact(zone, labels, labels_top, type, &ttlrdata); // in zdb_zone_record_delete
+}
+
+/**
+ * @brief Removes a record from a zone
+ *
+ * Copies the record then removes it from the zone and frees its memory.
+ * This allows removing safely a record using itself as a parameter.
+ *
+ * @param[in] zone the zone
+ * @param[in] labels the stack of labels of the dns name
+ * @param[in] labels_top the index of the top of the stack (the level)
+ * @param[in] type the type of the record
+ * @param[in] ttlrdata the ttl and rdata of the record.  NOTE: the caller stays the owner
+ */
+
+ya_result
+zdb_zone_record_delete_self(zdb_zone *zone, dnslabel_vector_reference labels, s32 labels_top, u16 type, zdb_packed_ttlrdata* packed_ttlrdata)
+{
+    u8 *tmp;
+    ya_result ret;
+    u8 tmp_[512];
+    zdb_ttlrdata ttlrdata;
+    ttlrdata.next = NULL;
+    ttlrdata.rdata_size = ZDB_PACKEDRECORD_PTR_RDATASIZE(packed_ttlrdata);
+    if(ttlrdata.rdata_size <= sizeof(tmp_))
+    {
+        tmp = &tmp_[0];
+    }
+    else   
+    {
+        MALLOC_OR_DIE(u8*,tmp,ttlrdata.rdata_size,GENERIC_TAG);
+    }
+    memcpy(tmp, ZDB_PACKEDRECORD_PTR_RDATAPTR(packed_ttlrdata), ttlrdata.rdata_size);
+    ttlrdata.rdata_pointer = tmp;
+    ttlrdata.ttl = packed_ttlrdata->ttl;
+
+    ret = zdb_rr_label_delete_record_exact(zone, labels, labels_top, type, &ttlrdata); // safe
+    
+    if(ttlrdata.rdata_size > sizeof(tmp_))
+    {
+        free(tmp); // scan-build false positive : this is only called because tmp has been mallocated
+    }
+    
+    return ret;
 }
 
 /**
@@ -274,6 +354,10 @@ zdb_zone_create(const u8* origin)
     u32 zone_footprint = zdb_zone_get_struct_size(origin);
     ZALLOC_ARRAY_OR_DIE(zdb_zone*, zone, zone_footprint, ZDB_ZONETAG);
     
+#ifdef DEBUG
+    memset(zone, 0xac, zone_footprint);
+#endif
+    
 #if HAS_TRACK_ZONES_DEBUG_SUPPORT
     smp_int_inc(&g_zone_instanciated_count);
     pthread_mutex_lock(&g_zone_instanciated_count.mutex);
@@ -296,11 +380,12 @@ zdb_zone_create(const u8* origin)
     /* zone->axfr_serial = 0; implicit */
 
 #if ZDB_HAS_DNSSEC_SUPPORT != 0
-    ZEROMEMORY(&zone->nsec, sizeof (nsec_zone_union));
+    ZEROMEMORY(&zone->nsec, sizeof(nsec_zone_union));
     zone->sig_validity_interval_seconds = 30*24*3600;       /* 1 month */
     zone->sig_validity_regeneration_seconds = 7*24*3600;    /* 1 week */
     zone->sig_validity_jitter_seconds = 86400;              /* 1 day */
     zone->sig_quota = 100;
+
 #endif
 
     zone->alarm_handle = alarm_open(zone->origin);
@@ -310,20 +395,23 @@ zdb_zone_create(const u8* origin)
 
     zone->query_access_filter = zdb_default_query_access_filter;
     zone->extension = NULL;
+#if ZDB_HAS_DNSSEC_SUPPORT
+    zone->sig_last_processed_node = NULL;
+#endif
     mutex_init(&zone->lock_mutex);
     cond_init(&zone->lock_cond);
     zone->rc = 1;
     zone->lock_owner = ZDB_ZONE_MUTEX_NOBODY;
     zone->lock_count = 0;
     zone->lock_reserved_owner = ZDB_ZONE_MUTEX_NOBODY;
+    zone->status = 0;
 #if DNSCORE_HAS_MUTEX_DEBUG_SUPPORT
     zone->lock_trace = NULL;
     zone->lock_id = 0;
     zone->lock_timestamp = 0;
 #endif
     zone->journal = NULL;
-    zone->sig_last_processed_node = NULL;
-        
+    
     return zone;
 }
 
@@ -426,7 +514,7 @@ zdb_zone_destroy(zdb_zone *zone)
         
         if(zone->journal != NULL)
         {
-            journal *jh = zone->journal;
+            journal *jh = zone->journal; // pointed for closing/releasing
             zdb_zone_unlock(zone, ZDB_ZONE_MUTEX_DESTROY);
             journal_close(jh); // only authorised usage of this call
             zone->journal = NULL;
@@ -467,6 +555,14 @@ zdb_zone_destroy(zdb_zone *zone)
         u32 zone_footprint = zdb_zone_get_struct_size(zone->origin);
         
         dnsname_zfree(zone->origin);
+        
+#if HAS_DNSSEC_SUPPORT
+        if(zone->sig_last_processed_node != NULL)
+        {
+            ZFREE_STRING(zone->sig_last_processed_node);
+            zone->sig_last_processed_node = NULL;
+        }
+#endif
 
 #ifdef DEBUG
         zone->origin = NULL;
@@ -501,7 +597,7 @@ zdb_zone_getsoa(const zdb_zone *zone, soa_rdata* soa_out)
     {
         log_err("zdb_zone_getsoa called on an unlocked zone: %{dnsname}", zone->origin);
         debug_log_stacktrace(MODULE_MSG_HANDLE, LOG_ERR, "zdb_zone_getsoa");
-        logger_flush();
+        //logger_flush();
     }
     else
     {
@@ -685,6 +781,238 @@ zdb_zone_isinvalid(zdb_zone *zone)
     return invalid;
 }
 
+#if HAS_DNSSEC_SUPPORT
+
+/**
+ * Adds a DNSKEY record in a zone from the dnssec_key object.
+ * 
+ * @param key
+ * @return TRUE iff the record has been added
+ */
+
+bool
+zdb_zone_add_dnskey_from_key(zdb_zone *zone, const dnssec_key *key)
+{
+    yassert(zdb_zone_islocked(zone));
+    
+    zdb_packed_ttlrdata *dnskey_record;
+    u32 rdata_size = key->vtbl->dnskey_key_rdatasize(key);
+    ZDB_RECORD_ZALLOC_EMPTY(dnskey_record, 86400, rdata_size);
+    key->vtbl->dnskey_key_writerdata(key, ZDB_PACKEDRECORD_PTR_RDATAPTR(dnskey_record));
+
+    // store the record
+
+    if(zdb_record_insert_checked(&zone->apex->resource_record_set, TYPE_DNSKEY, dnskey_record)) /* FB done */
+    {
+        zdb_ttlrdata unpacked_dnskey_record;
+        unpacked_dnskey_record.rdata_pointer = ZDB_PACKEDRECORD_PTR_RDATAPTR(dnskey_record);
+        unpacked_dnskey_record.rdata_size = ZDB_PACKEDRECORD_PTR_RDATASIZE(dnskey_record);
+        unpacked_dnskey_record.ttl = dnskey_record->ttl;
+        zdb_listener_notify_add_record(zone, zone->origin_vector.labels, zone->origin_vector.size, TYPE_DNSKEY, &unpacked_dnskey_record);
+        
+        return TRUE;
+    }
+    else
+    {
+        ZDB_RECORD_ZFREE(dnskey_record);
+        
+        return FALSE;
+    }
+}
+
+/**
+ * Removes a DNSKEY record in a zone from the dnssec_key object.
+ * 
+ * @param key
+ * @return TRUE iff the record has been found and removed
+ */
+
+
+bool
+zdb_zone_remove_dnskey_from_key(zdb_zone *zone, const dnssec_key *key)
+{
+    yassert(zdb_zone_islocked(zone));
+    
+    zdb_packed_ttlrdata *dnskey_record;
+    u32 rdata_size = key->vtbl->dnskey_key_rdatasize(key);
+    ZDB_RECORD_ZALLOC_EMPTY(dnskey_record, 86400, rdata_size);
+    key->vtbl->dnskey_key_writerdata(key, ZDB_PACKEDRECORD_PTR_RDATAPTR(dnskey_record));
+
+    zdb_ttlrdata unpacked_dnskey_record;
+    unpacked_dnskey_record.rdata_pointer = ZDB_PACKEDRECORD_PTR_RDATAPTR(dnskey_record);
+    unpacked_dnskey_record.rdata_size = ZDB_PACKEDRECORD_PTR_RDATASIZE(dnskey_record);
+    unpacked_dnskey_record.ttl = dnskey_record->ttl;
+    
+    // remove the record
+
+    if(zdb_record_delete_self_exact(&zone->apex->resource_record_set, TYPE_DNSKEY, &unpacked_dnskey_record) >= 0)
+    {
+        zdb_listener_notify_remove_record(zone, zone->origin, TYPE_DNSKEY, &unpacked_dnskey_record);
+        ZDB_RECORD_ZFREE(dnskey_record);
+        return TRUE;
+    }
+    else
+    {    
+        ZDB_RECORD_ZFREE(dnskey_record);
+        return FALSE;
+    }
+}
+
+/**
+ * 
+ * Returns TRUE iff the key is present as a record in the zone
+ * 
+ * @param zone
+ * @param key
+ * @return 
+ */
+
+bool
+zdb_zone_contains_dnskey_record_for_key(zdb_zone *zone, const dnssec_key *key)
+{
+    yassert(zdb_zone_islocked(zone));
+    
+    const zdb_packed_ttlrdata *dnskey_rrset = zdb_record_find(&zone->apex->resource_record_set, TYPE_DNSKEY);
+    
+    const zdb_packed_ttlrdata *dnskey_record = dnskey_rrset;
+    
+    while(dnskey_record != NULL)
+    {
+        if(dnskey_matches_rdata(key, ZDB_PACKEDRECORD_PTR_RDATAPTR(dnskey_record), ZDB_PACKEDRECORD_PTR_RDATASIZE(dnskey_record)))
+        {
+            return TRUE;
+        }
+
+        dnskey_record = dnskey_record->next;
+    }
+    
+    return FALSE;
+}
+
+/**
+ * Returns TRUE iff there is at least one RRSIG record with the tag and algorithm of the key
+ * 
+ * @param zone
+ * @param key
+ * @return 
+ */
+
+bool
+zdb_zone_apex_contains_rrsig_record_by_key(zdb_zone *zone, const dnssec_key *key)
+{
+    yassert(zdb_zone_islocked(zone));
+    
+    const zdb_packed_ttlrdata *rrsig_rrset = zdb_record_find(&zone->apex->resource_record_set, TYPE_RRSIG);
+    
+    if(rrsig_rrset != NULL)
+    {
+        const zdb_packed_ttlrdata *rrsig_record = rrsig_rrset;
+        u16 tag = dnssec_key_get_tag_const(key);
+        u8 algorithm = dnssec_key_get_algorithm(key);
+        
+        while(rrsig_record != NULL)
+        {
+            if((RRSIG_ALGORITHM(rrsig_record) == algorithm) && (RRSIG_KEY_TAG(rrsig_record) == tag))
+            {
+                return TRUE;
+            }
+
+            rrsig_record = rrsig_record->next;
+        }
+    }
+    
+    return FALSE;
+}
+
+void
+zdb_zone_update_keystore_keys_from_zone(zdb_zone *zone)
+{
+    // keystore keys with a publish time that did not expire yet have to be added
+    // keystore keys with an unpublish time that passed have to be removed
+    //
+    // after (and only after) the signature is done, set alarms at all the (relevant) timings of the keys (publish, activate, inactivate, unpublish)
+    
+    yassert(zdb_zone_islocked(zone));
+
+    ptr_vector dnskey_add = EMPTY_PTR_VECTOR;
+    ptr_vector dnskey_del = EMPTY_PTR_VECTOR;
+
+    for(int i = 0; ; ++i)
+    {
+        dnssec_key *key = dnssec_keystore_acquire_key(zone->origin, i);
+        if(key == NULL)
+        {
+            break;
+        }
+
+        if(dnskey_is_published(key, time(NULL)))
+        {
+            if(!zdb_zone_contains_dnskey_record_for_key(zone, key))
+            {
+                dnskey_acquire(key);
+                ptr_vector_append(&dnskey_add, key);
+            }
+        }
+
+        dnskey_release(key);
+    }
+
+
+    zdb_packed_ttlrdata *dnskey_rrset = zdb_record_find(&zone->apex->resource_record_set, TYPE_DNSKEY);
+    zdb_packed_ttlrdata *dnskey_record = dnskey_rrset;
+    while(dnskey_record != NULL)
+    {
+        dnssec_key *key;
+
+        if(ISOK(dnssec_keystore_load_private_key_from_rdata(
+                ZDB_PACKEDRECORD_PTR_RDATAPTR(dnskey_record),
+                ZDB_PACKEDRECORD_PTR_RDATASIZE(dnskey_record),
+                zone->origin,
+                &key)))
+        {
+            if(dnskey_is_unpublished(key, time(NULL)))
+            {
+                // need to unpublish
+                ptr_vector_append(&dnskey_del, dnskey_record);
+            }
+
+            dnskey_release(key);
+        }
+
+        dnskey_record = dnskey_record->next;
+    }
+
+    if(ptr_vector_size(&dnskey_add) + ptr_vector_size(&dnskey_del) > 0)
+    {
+        zdb_icmtl icmtl;
+        dnsname_vector apex_name;
+        DEBUG_RESET_dnsname(apex_name);
+        dnsname_to_dnsname_vector(zone->origin, &apex_name);
+        ya_result ret;
+
+        if(ISOK(ret = zdb_icmtl_begin(&icmtl, zone)))
+        {
+            for(int i = 0; i <= ptr_vector_last_index(&dnskey_del); ++i)
+            {
+                zdb_packed_ttlrdata *dnskey_record = (zdb_packed_ttlrdata*)ptr_vector_get(&dnskey_del, i);
+
+                zdb_zone_record_delete_self(zone, apex_name.labels, -1, TYPE_DNSKEY, dnskey_record);
+            }
+
+            for(int i = 0; i <= ptr_vector_last_index(&dnskey_add); ++i)
+            {
+                dnssec_key *key = (dnssec_key*)ptr_vector_get(&dnskey_add, i);
+                zdb_zone_add_dnskey_from_key(zone, key);
+            }
+
+            zdb_icmtl_end(&icmtl);
+        }
+    }
+        
+}
+
+#endif
+
 #ifdef DEBUG
 
 /**
