@@ -1,36 +1,36 @@
 /*------------------------------------------------------------------------------
-*
-* Copyright (c) 2011-2016, EURid. All rights reserved.
-* The YADIFA TM software product is provided under the BSD 3-clause license:
-* 
-* Redistribution and use in source and binary forms, with or without 
-* modification, are permitted provided that the following conditions
-* are met:
-*
-*        * Redistributions of source code must retain the above copyright 
-*          notice, this list of conditions and the following disclaimer.
-*        * Redistributions in binary form must reproduce the above copyright 
-*          notice, this list of conditions and the following disclaimer in the 
-*          documentation and/or other materials provided with the distribution.
-*        * Neither the name of EURid nor the names of its contributors may be 
-*          used to endorse or promote products derived from this software 
-*          without specific prior written permission.
-*
-* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
-* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
-* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
-* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
-* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
-* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
-* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
-* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
-* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
-* POSSIBILITY OF SUCH DAMAGE.
-*
-*------------------------------------------------------------------------------
-*
-*/
+ *
+ * Copyright (c) 2011-2016, EURid. All rights reserved.
+ * The YADIFA TM software product is provided under the BSD 3-clause license:
+ * 
+ * Redistribution and use in source and binary forms, with or without 
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ *        * Redistributions of source code must retain the above copyright 
+ *          notice, this list of conditions and the following disclaimer.
+ *        * Redistributions in binary form must reproduce the above copyright 
+ *          notice, this list of conditions and the following disclaimer in the 
+ *          documentation and/or other materials provided with the distribution.
+ *        * Neither the name of EURid nor the names of its contributors may be 
+ *          used to endorse or promote products derived from this software 
+ *          without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+ * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
+ * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
+ * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
+ * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
+ * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
+ * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+ * POSSIBILITY OF SUCH DAMAGE.
+ *
+ *------------------------------------------------------------------------------
+ *
+ */
 /** @defgroup yadifad Yet Another DNS Implementation for all
  * 
  *  @brief Yet Another DNS Implementation for all
@@ -72,8 +72,11 @@
 
 #include <dnscore/sys_get_cpu_count.h>
 
+#if ZDB_HAS_DNSSEC_SUPPORT
 #include <dnsdb/dnssec.h>
-#include <dnsdb/dnssec_keystore.h>
+#include <dnsdb/dnssec-keystore.h>
+#include <dnsdb/zdb.h>
+#endif
 
 #include "server_error.h"
 #include "config_error.h"
@@ -81,16 +84,13 @@
 #include "server.h"
 #include "notify.h"
 
+
 #include "database-service.h"
 
 #if HAS_DYNUPDATE_SUPPORT
 #include "dynupdate_query_service.h"
 #endif
 
-#if HAS_DYNCONF_SUPPORT
-#include "dynconf.h"
-#endif
-
 #include "buildinfo.h"
 
 #define MODULE_MSG_HANDLE g_server_logger
@@ -102,6 +102,8 @@ static bool server_do_clean_exit = FALSE;
 int g_yadifa_exitcode = EXIT_SUCCESS;
 static bool own_pid = FALSE;
 
+static bool main_config_log_from_start = FALSE; // start logging asap
+
 void config_logger_setdefault();
 void config_logger_cleardefault();
 
@@ -193,11 +195,11 @@ yadifad_config_on_section_loggers_read(const char* name, int index)
 {
     //formatln("yadifad_config_on_section_main_read(%s,%i)", name, index);
 
-    ya_result                                                   return_code;
+    ya_result                                                   ret;
     
-    if(FAIL(return_code = pid_check_running_program(PROGRAM_NAME, g_config->pid_file))) /// @todo needs to add pid_file stuff
+    if(FAIL(ret = pid_check_running_program(PROGRAM_NAME, g_config->pid_file))) /// @todo 20160127 edf -- needs to add pid_file stuff
     {
-        return return_code;
+        return ret;
     }
     
     /*
@@ -215,12 +217,16 @@ yadifad_config_on_section_loggers_read(const char* name, int index)
     {
         config_logger_setdefault();
     }
+    else
+    {
+        config_logger_cleardefault();
+    }
     
     main_dump_info();
     
-    if(FAIL(return_code = config_update_network(g_config)))
+    if(FAIL(ret = server_service_init()))
     {
-        return return_code;
+        return ret;
     }
     
     database_service_init();
@@ -229,19 +235,19 @@ yadifad_config_on_section_loggers_read(const char* name, int index)
      * quitting the program
      */
 
-    if(FAIL(return_code = signal_handler_init()))
+    if(FAIL(ret = signal_handler_init()))
     {
-        log_err("failed to setup the signal handler: %r", return_code);
+        log_err("failed to setup the signal handler: %r", ret);
 
         if(!(g_config->server_flags & SERVER_FL_DAEMON))
         {
-            osformatln(termerr, "error: failed to setup the signal handler: %r", return_code);
+            osformatln(termerr, "error: failed to setup the signal handler: %r", ret);
             flusherr();
         }
 
         logger_flush();
 
-        return return_code;
+        return ret;
     }
 
     notify_service_init();
@@ -263,15 +269,21 @@ yadifad_config_on_section_loggers_read(const char* name, int index)
 int
 main_config(int argc, char *argv[])
 {
-    ya_result return_code;
+    ya_result ret;
+    
+    
+    if(main_config_log_from_start)
+    {
+        config_logger_setdefault();
+    }
     
     /*
      *  Initialise configuration file and set standard values
      */
         
-    if(FAIL(return_code = yadifad_config_init()))
+    if(FAIL(ret = yadifad_config_init()))
     {
-        osformatln(termerr, "error: %r", return_code);
+        osformatln(termerr, "error: %r", ret);
         flusherr();
 
         return ERROR;
@@ -280,9 +292,9 @@ main_config(int argc, char *argv[])
     // channels then loggers
     config_add_on_section_read_callback("loggers", yadifad_config_on_section_loggers_read);
     
-    if((return_code = yadifad_config_cmdline(argc, argv)) != 0)
+    if((ret = yadifad_config_cmdline(argc, argv)) != 0)
     {
-        if(FAIL(return_code))
+        if(FAIL(ret))
         {
             return ERROR;
         }
@@ -290,17 +302,17 @@ main_config(int argc, char *argv[])
         return 1;
     }
 
-    if(FAIL(return_code = yadifad_config_read(g_config->config_file)))
+    if(FAIL(ret = yadifad_config_read(g_config->config_file)))
     {
-        osformatln(termerr, "error: %r", return_code);
+        osformatln(termerr, "error: %r", ret);
         flusherr();
 
         return ERROR;
     }
     
-    if(FAIL(return_code = yadifad_config_finalise()))
+    if(FAIL(ret = yadifad_config_finalise()))
     {
-        osformatln(termerr, "error: %r", return_code);
+        osformatln(termerr, "error: %r", ret);
         flusherr();
 
         return ERROR;
@@ -391,23 +403,15 @@ main_exit()
         log_info("shutting down");
     }
     
+    server_service_stop();
+    
+    server_service_finalise();
+    
 #if HAS_DYNUPDATE_SUPPORT
     dynupdate_query_service_stop();
 #endif
     
     notify_service_stop();
-        
-    if(own_pid)
-    {
-        log_info("releasing pid file lock");
-        
-        pid_file_destroy(g_config->pid_file);
-    }
-    
-    logger_flush();
-        
-    flushout();
-    flusherr();
     
     signal_handler_finalise();
     notify_service_finalise();
@@ -424,12 +428,38 @@ main_exit()
         
         database_finalize();
         
+        if(own_pid)
+        {
+            log_info("releasing pid file lock");
+
+            pid_file_destroy(g_config->pid_file);
+        }
+
+        logger_flush();
+
+        flushout();
+        flusherr();
+
 #if HAS_ACL_SUPPORT
         acl_free_definitions();
 #endif
         
         dnscore_finalize();
     }
+    else
+    {
+        if(own_pid)
+        {
+            log_info("releasing pid file lock");
+
+            pid_file_destroy(g_config->pid_file);
+        }
+
+        logger_flush();
+
+        flushout();
+        flusherr();
+    }
 }
 
 /** \brief Main function of yadifa
@@ -443,14 +473,67 @@ main_exit()
  *
  */
 
+
+
+/**
+ * This will stop YADIFAD if the libraries have been build-configured differently or made differently.
+ * 
+ * The most typical trigger is "make debug" vs "make", is, because it has not been followed by,
+ * respectively, a "make debug-install" or a "make install"
+ * 
+ * Or, in the case of a static build, a mix code versions that would have required a "make clean"
+ * 
+ */
+
+static void
+main_check_build_settings()
+{
+    if(dnscore_getfingerprint() != dnscore_getmyfingerprint())
+    {
+        printf("yadifad: the linked dnscore features are %08x but the the lib has been compiled against one with %08x", dnscore_getfingerprint(), dnscore_getmyfingerprint());
+        fflush(NULL);
+        exit(-1);
+    }
+    
+    if(dnsdb_getfingerprint() != dnsdb_getmyfingerprint())
+    {
+        printf("yadifad: the linked dnsdb features are %08x but the the lib has been compiled against one with %08x", dnsdb_getfingerprint(), dnsdb_getmyfingerprint());
+        fflush(NULL);
+        exit(-1);
+    }
+}
+
+/**
+ * The flag must be checked this way as the internal command line/configuration parsing mechanism
+ * would miss the start.
+ * 
+ * @param argc
+ * @param argv
+ */
+
+static void
+main_check_lock_from_start(int argc, const char **argv)
+{
+    for(int i = 1; i < argc; ++i)
+    {
+        if(strcmp(argv[i], "-L") == 0)
+        {
+            main_config_log_from_start = TRUE;
+        }
+    }
+}
+
 int
 main(int argc, char *argv[])
 {
-    ya_result                                                   return_code;
+    ya_result ret;
+    
+
 
     /*    ------------------------------------------------------------    */
     
-    /* Initializes the core library:
+    /**
+     *  Initialises the core library:
      * _ checks basic architecture settings (endianness, types sizes, random generator, ...)
      * _ initialises dns types and classes name<->id matching
      * _ initialises text formatting (format*, log*)
@@ -461,9 +544,17 @@ main(int argc, char *argv[])
      * _ registers an exit function
      * _ resets and start the alarm/timer function
      */
+    
+#ifdef DEBUG
+    puts("YADIFA debug build");
+#endif
+    
+    main_check_build_settings();
         
     dnscore_init();
     
+    main_check_lock_from_start(argc, (const char**)argv);
+        
     async_message_pool_init();
 
     // registers yadifad error codes
@@ -474,23 +565,19 @@ main(int argc, char *argv[])
 
     atexit(main_exit);
     
-#if HAS_DYNCONF_SUPPORT
-    //dynconf_service_init();
-    //dynconf_service_start();
-#endif    
     // configures, exit if ordered to (version/help or error)
     
-    if((return_code = main_config(argc, argv)) != SUCCESS)
+    if((ret = main_config(argc, argv)) != SUCCESS)
     {
-        return ISOK(return_code)?EXIT_SUCCESS:EXIT_FAILURE;
+        return ISOK(ret)?EXIT_SUCCESS:EXIT_FAILURE;
     }
 
 
 
     // This is always 'exit' on failure
-    if(FAIL(return_code = pid_check_running_program(PROGRAM_NAME, g_config->pid_file)))
+    if(FAIL(ret = pid_check_running_program(PROGRAM_NAME, g_config->pid_file)))
     {
-        return return_code;
+        return ret;
     }
 
     /*
@@ -505,21 +592,23 @@ main(int argc, char *argv[])
 
     u32 setup_flags = SETUP_CORE_LIMITS | SETUP_ID_CHANGE | SETUP_CREATE_PID_FILE;
 
-    if (g_config->server_flags & SERVER_FL_CHROOT)
+    if(g_config->server_flags & SERVER_FL_CHROOT)
     {
         setup_flags |= SETUP_ROOT_CHANGE;
     }
 
-    if(FAIL(return_code = server_setup_env(&g_config->pid, &g_config->pid_file, g_config->uid, g_config->gid, setup_flags)))
+    if(FAIL(ret = server_setup_env(&g_config->pid, &g_config->pid_file, g_config->uid, g_config->gid, setup_flags)))
     {
-        log_err("server setup failed: %r", return_code);
+        log_err("server setup failed: %r", ret);
         return EXIT_FAILURE;
     }
     
     own_pid = TRUE;
     
+#if ZDB_HAS_DNSSEC_SUPPORT
     dnssec_keystore_setpath(g_config->keys_path);
     dnssec_set_xfr_path(g_config->xfr_path);
+#endif
     logger_reopen();
 
     /// last tests before the real startup (directory writable)
@@ -535,15 +624,15 @@ main(int argc, char *argv[])
 
     log_info("loading zones");
     
-    if(FAIL(return_code = database_startup(&g_config->database)))
+    if(FAIL(ret = database_startup(&g_config->database)))
     {
-        log_err("loading zones: %r", return_code);
+        log_err("loading zones: %r", ret);
 
         return EXIT_FAILURE;
     }
     
     /**
-     * @todo only do this if we are master for at least one zone
+     * @todo 20140415 edf -- only do this if we are master for at least one zone
      */
     
     log_info("starting notify service");
@@ -565,16 +654,20 @@ main(int argc, char *argv[])
      */
     
     int exit_code;
-    
-    if(ISOK(server_run()))
-    {
-        exit_code = EXIT_SUCCESS;
-    }
-    else
-    {
-        exit_code = EXIT_FAILURE;
-    }
 
+
+        
+        if(ISOK(ret = server_service_start_and_wait()))
+        {
+            exit_code = EXIT_SUCCESS;
+        }
+        else
+        {
+            exit_code = EXIT_FAILURE;
+        }
+
+
+    
     /// @note DO NOT: logger_finalize() don't, it will be done automatically at exit
 
     return exit_code;
