@@ -1,36 +1,36 @@
 /*------------------------------------------------------------------------------
-*
-* Copyright (c) 2011-2016, EURid. All rights reserved.
-* The YADIFA TM software product is provided under the BSD 3-clause license:
-* 
-* Redistribution and use in source and binary forms, with or without 
-* modification, are permitted provided that the following conditions
-* are met:
-*
-*        * Redistributions of source code must retain the above copyright 
-*          notice, this list of conditions and the following disclaimer.
-*        * Redistributions in binary form must reproduce the above copyright 
-*          notice, this list of conditions and the following disclaimer in the 
-*          documentation and/or other materials provided with the distribution.
-*        * Neither the name of EURid nor the names of its contributors may be 
-*          used to endorse or promote products derived from this software 
-*          without specific prior written permission.
-*
-* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
-* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
-* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
-* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
-* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
-* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
-* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
-* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
-* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
-* POSSIBILITY OF SUCH DAMAGE.
-*
-*------------------------------------------------------------------------------
-*
-*/
+ *
+ * Copyright (c) 2011-2016, EURid. All rights reserved.
+ * The YADIFA TM software product is provided under the BSD 3-clause license:
+ * 
+ * Redistribution and use in source and binary forms, with or without 
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ *        * Redistributions of source code must retain the above copyright 
+ *          notice, this list of conditions and the following disclaimer.
+ *        * Redistributions in binary form must reproduce the above copyright 
+ *          notice, this list of conditions and the following disclaimer in the 
+ *          documentation and/or other materials provided with the distribution.
+ *        * Neither the name of EURid nor the names of its contributors may be 
+ *          used to endorse or promote products derived from this software 
+ *          without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+ * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
+ * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
+ * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
+ * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
+ * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
+ * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+ * POSSIBILITY OF SUCH DAMAGE.
+ *
+ *------------------------------------------------------------------------------
+ *
+ */
 /** @defgroup server
  *  @ingroup yadifad
  *  @brief server
@@ -58,6 +58,10 @@ extern logger_handle *g_server_logger;
 #include "process_class_ch.h"
 #include "confs.h"
 
+#define CHVRSION_TAG 0x4e4f495352564843
+#define CHHOSTNM_TAG 0x4d4e54534f484843
+#define CHIDSVR_TAG 0x52565344494843
+
 extern logger_handle* g_server_logger;
 
 /*
@@ -80,7 +84,7 @@ class_ch_set_hostname(const char *name)
     {
         size_t name_len = MIN(strlen(name), 255);
         u8* tmp;
-        MALLOC_OR_DIE(u8*, tmp, 13 + name_len, GENERIC_TAG);
+        MALLOC_OR_DIE(u8*, tmp, 13 + name_len, CHHOSTNM_TAG);
         memcpy(tmp, chaos_txt_stub, 10);
         SET_U16_AT(tmp[10], htons(name_len + 1));
         tmp[12] = (u8)name_len;
@@ -104,7 +108,7 @@ class_ch_set_version(const char *name)
     {
         size_t name_len = MIN(strlen(name), 255);
         u8* tmp;
-        MALLOC_OR_DIE(u8*, tmp, 13 + name_len, GENERIC_TAG);
+        MALLOC_OR_DIE(u8*, tmp, 13 + name_len, CHVRSION_TAG);
         memcpy(tmp, chaos_txt_stub, 10);
         SET_U16_AT(tmp[10], htons(name_len + 1));
         tmp[12] = (u8)name_len;
@@ -128,7 +132,7 @@ class_ch_set_id_server(const char *name)
     {
         size_t name_len = MIN(strlen(name), 255);
         u8* tmp;
-        MALLOC_OR_DIE(u8*, tmp, 13 + name_len, GENERIC_TAG);
+        MALLOC_OR_DIE(u8*, tmp, 13 + name_len, CHIDSVR_TAG);
         memcpy(tmp, chaos_txt_stub, 10);
         SET_U16_AT(tmp[10], htons(name_len + 1));
         tmp[12] = (u8)name_len;
@@ -226,7 +230,7 @@ chaos_make_message(message_data *mesg, const u8* record_wire, u32 record_wire_le
         p[ 7] = rcode_ext >> 8;
         p[ 8] = rcode_ext;
 
-#if HAS_NSID_SUPPORT
+#if DNSCORE_HAS_NSID_SUPPORT
         if(!mesg->nsid)
         {
             p[ 9] = 0;
@@ -274,7 +278,7 @@ class_ch_process(message_data *mesg)
     purd.packet_size = mesg->received;
     purd.offset = DNS_HEADER_LENGTH;
 
-    if(FAIL(return_value = packet_reader_read_fqdn(&purd, qname, sizeof (qname))))
+    if(FAIL(return_value = packet_reader_read_fqdn(&purd, qname, sizeof(qname))))
     {
         /* oops */
 
