@@ -1,53 +1,57 @@
 /*------------------------------------------------------------------------------
-*
-* Copyright (c) 2011-2016, EURid. All rights reserved.
-* The YADIFA TM software product is provided under the BSD 3-clause license:
-* 
-* Redistribution and use in source and binary forms, with or without 
-* modification, are permitted provided that the following conditions
-* are met:
-*
-*        * Redistributions of source code must retain the above copyright 
-*          notice, this list of conditions and the following disclaimer.
-*        * Redistributions in binary form must reproduce the above copyright 
-*          notice, this list of conditions and the following disclaimer in the 
-*          documentation and/or other materials provided with the distribution.
-*        * Neither the name of EURid nor the names of its contributors may be 
-*          used to endorse or promote products derived from this software 
-*          without specific prior written permission.
-*
-* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
-* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
-* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
-* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
-* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
-* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
-* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
-* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
-* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
-* POSSIBILITY OF SUCH DAMAGE.
-*
-*------------------------------------------------------------------------------
-*
-*/
+ *
+ * Copyright (c) 2011-2016, EURid. All rights reserved.
+ * The YADIFA TM software product is provided under the BSD 3-clause license:
+ * 
+ * Redistribution and use in source and binary forms, with or without 
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ *        * Redistributions of source code must retain the above copyright 
+ *          notice, this list of conditions and the following disclaimer.
+ *        * Redistributions in binary form must reproduce the above copyright 
+ *          notice, this list of conditions and the following disclaimer in the 
+ *          documentation and/or other materials provided with the distribution.
+ *        * Neither the name of EURid nor the names of its contributors may be 
+ *          used to endorse or promote products derived from this software 
+ *          without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+ * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
+ * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
+ * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
+ * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
+ * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
+ * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+ * POSSIBILITY OF SUCH DAMAGE.
+ *
+ *------------------------------------------------------------------------------
+ *
+ */
 
 /**
  *  @defgroup server Server
  *  @ingroup yadifad
- *  @brief Single threaded server. This one has the best performance on all of our setups with kernels <= 2.6.32
- *
- *  This is the default and best server available for Yadifa.
- *  It works like this:
- *  A select is made on all network file descriptors with a timeout of 1 second.
- *  Each usable descriptor is then processed.
- *  _ UDP are single threaded (receive, process, send)
- *  _ TCP are multithreaded (accept, thread(process & send))
- *  Then the scheduler tasks are processed.
+ *  @brief multithreaded server
+ * 
+ *  Multiples threads for UDP on the same socket per interface.
+ *  One thread per interface for TCP, dispatching accepts to worker threads.
  *
+ *  One weakness: long locks on the DB may lead to lost messages
+ * 
+ *  known case: an incremental update of 4MB triggers a mpcl pf the database for 110ms on a middle range server
+ *              every message sent in this window after the ~256 + workers will be lost.
+ *              This case has been tested by another TLD.
+ * 
+ *  to relativise: a typical update locks the database for less than a millisecond.
+ *              a throughput of about 256000 queries per second would be needed to miss the first message.
+ *              A typical server has to answer to about 10000 to 20000 queries per second, peak use.
+ * 
  * @{
  */
-/*----------------------------------------------------------------------------*/
 
 #define SERVER_ST_C_
 
@@ -75,6 +79,12 @@
 extern logger_handle *g_server_logger;
 #define MODULE_MSG_HANDLE g_server_logger
 
+// DEBUG build: log debug 5 of incoming wire
+#define DUMP_UDP_MT_RECEIVED_WIRE 0
+
+// DEBUG build: log debug 5 of outgoing wire
+#define DUMP_UDP_MT_OUTPUT_WIRE 0
+
 #include "server-mt.h"
 
 #include "server_context.h"
@@ -122,7 +132,6 @@ extern logger_handle *g_server_logger;
 static server_statistics_t server_statistics_sum;
 
 #define SYNCED_THREAD_STATUS_TERMINATED 1
-#define PAUSE_ALL_ON_TASK               0
 #define SERVER_MAX_UDP_THREADS          255
 
 /**
@@ -131,14 +140,15 @@ static server_statistics_t server_statistics_sum;
  * The syncs are slow, but it does not matter.
  */
 
+#define SYNTHRDT_TAG 0x54445248544e5953
+
 struct synced_thread_t
 {
-    interface *intf;
+    //interface *intf;
     pthread_t id;
+    int fdsock;
     u16 idx;
-#if PAUSE_ALL_ON_TASK != 0
-    volatile u32 paused;
-#endif
+
     volatile u8  status;
     
     message_data *udp_mesg;
@@ -158,9 +168,7 @@ struct synced_threads_t
     synced_thread_t* threads;    
     mutex_t mtx;
     u32 thread_count;
-#if PAUSE_ALL_ON_TASK != 0
-    volatile u32 pause;
-#endif
+
     volatile bool terminate;
 };
 
@@ -174,15 +182,13 @@ synced_init(u32 count)
     ZEROMEMORY(&synced_threads, sizeof(synced_threads));
     mutex_init(&synced_threads.mtx);
     
-    MALLOC_OR_DIE(synced_thread_t*, synced_threads.threads, count * sizeof(synced_thread_t), GENERIC_TAG);
+    MALLOC_OR_DIE(synced_thread_t*, synced_threads.threads, count * sizeof(synced_thread_t), SYNTHRDT_TAG);
     ZEROMEMORY(synced_threads.threads, count * sizeof(synced_thread_t));
     
     for(u32 t = 0; t < count; t++)
     {
         synced_threads.threads[t].id = 0;
-#if PAUSE_ALL_ON_TASK != 0
-        synced_threads.threads[t].paused = 0;
-#endif
+
         synced_threads.threads[t].idx = t;
         ZEROMEMORY(&synced_threads.threads[t].statistics, sizeof(server_statistics_t));
         MALLOC_OR_DIE(message_data*, synced_threads.threads[t].udp_mesg, sizeof(message_data), MESGDATA_TAG);
@@ -204,37 +210,6 @@ synced_finalize()
     ZEROMEMORY(&synced_threads, sizeof(synced_threads));
 }
 
-static bool
-synced_shouldpause()
-{
-#if PAUSE_ALL_ON_TASK != 0
-    return (synced_threads.pause & 1) != 0;
-#else
-    return FALSE;
-#endif
-}
-
-
-
-static void
-synced_wait(synced_thread_t *st)
-{
-#if PAUSE_ALL_ON_TASK != 0
-#ifdef DEBUG
-    log_debug("synced_wait: st %d/%d: waiting ... (%x)", st->idx+1, synced_threads.thread_count, st->paused);
-#endif
-    
-    while(((st->paused = synced_threads.pause) & 1) != 0)
-    {
-        usleep(1000); // 1ms
-    }
-    
-#ifdef DEBUG
-    log_debug("synced_wait: st %d/%d: resuming ... (%x)", st->idx+1, synced_threads.thread_count, st->paused);
-#endif
-#endif
-}
-
 static void
 synced_set_terminated(synced_thread_t *st)
 {
@@ -256,7 +231,19 @@ synced_stop()
     
     for(u32 i = 0; i < synced_threads.thread_count; i++)
     {
-        close_ex(synced_threads.threads[i].intf->udp.sockfd);
+        /*
+        close_ex(synced_threads.threads[i].fdsock);
+        close_ex(synced_threads.threads[i].fdsock);
+        */
+#ifdef DEBUG
+        int ret = pthread_kill(synced_threads.threads[i].id, SIGPIPE);
+        if(ret != 0)
+        {
+            log_err("could not send SIGPIPE signal to worker %p", synced_threads.threads[i].id);
+        }
+#else
+        pthread_kill(synced_threads.threads[i].id, SIGPIPE);
+#endif
     }
     
     /* wait everybody has stopped */
@@ -298,32 +285,6 @@ synced_stop()
     }
 }
 
-#if UDP_USE_MESSAGES
-
-/*
- * from: http://www.mombu.com/programming/c/t-how-to-get-udp-destination-address-on-incoming-packets-7784569.html
- */
-
-#if defined IP_RECVDSTADDR
-# define DSTADDR_SOCKOPT IP_RECVDSTADDR
-# define DSTADDR_DATASIZE (CMSG_SPACE(sizeof(struct in_addr)))
-# define dstaddr(x) (CMSG_DATA(x))
-#elif defined IP_PKTINFO
-# define DSTADDR_SOCKOPT IP_PKTINFO
-# define DSTADDR_DATASIZE (CMSG_SPACE(sizeof(struct in_pktinfo)))
-# define dstaddr(x) (&(((struct in_pktinfo *)(CMSG_DATA(x)))->ipi_addr))
-#else
-# error "can't determine socket option"
-#endif
-
-/*
-union cmsghdr_dstaddr {
-struct cmsghdr cmsg;
-u_char data[DSTADDR_DATASIZE];
-};
-*/
-#endif
-
 /*------------------------------------------------------------------------------
  * GLOBAL VARIABLES */
 
@@ -348,17 +309,9 @@ u_char data[DSTADDR_DATASIZE];
 
 
 static void
-server_mt_process_udp_update(zdb *database, synced_thread_t *st)
+server_mt_process_udp_update(zdb *database, message_data *mesg)
 {
-    struct msghdr *msghdr;
-    
-#if UDP_USE_MESSAGES
-    msghdr = &st->udp_msghdr;
-#else
-    msghdr = NULL;
-#endif
-    
-    dynupdate_query_service_enqueue(database, st->udp_mesg, msghdr);
+    dynupdate_query_service_enqueue(database, mesg);
 }
 
 #endif
@@ -382,38 +335,41 @@ server_mt_process_udp(zdb *database, synced_thread_t *st)
     server_statistics_t *local_statistics = &st->statistics;
     
     message_data *mesg = st->udp_mesg;
+    /*
+      On MESSAGES mode, this is already setup by the caller:
+      
+    st->udp_iovec.iov_base = mesg->buffer;
+    st->udp_iovec.iov_len = sizeof(mesg->buffer);
+    st->udp_msghdr.msg_name = &mesg->other.sa;
+    st->udp_msghdr.msg_namelen = sizeof(socketaddress);
+    st->udp_msghdr.msg_controllen = ANCILIARY_BUFFER_SIZE;
+    */
+    
+#if UDP_USE_MESSAGES
+    st->udp_iovec.iov_len = sizeof(mesg->buffer);
+#endif
     
-    int fd = mesg->sockfd;
-
     ssize_t n;
     
-    for(;;)
+    for(;;) // loop until reception, critical failure or shutdown
     {
-        while(synced_shouldpause())
-        {
-            // pause
-            synced_wait(st);
-        }
-
 #if !UDP_USE_MESSAGES
         
-        n = recvfrom(fd, mesg->buffer, sizeof(mesg->buffer), 0, (struct sockaddr*)&mesg->other.sa, &mesg->addr_len);
+
+        
+        n = recvfrom(st->fdsock, mesg->buffer, sizeof(mesg->buffer), 0, (struct sockaddr*)&mesg->other.sa, &mesg->addr_len);
         
         if(n >= 0)
         {
-#ifdef DEBUG
-            log_debug("server_mt_process_udp: recvfrom: got %d bytes from %{sockaddr}", n, &mesg->other.sa);
-            log_memdump_ex(g_server_logger, MSG_DEBUG5, mesg->buffer, n, 16, OSPRINT_DUMP_HEXTEXT);
-#endif
+
             break;
         }
         
         /*
-        * errno is not a variable but a macro
-        *
-        */
+         * errno is not a variable but a macro
+         */
         int err = errno;
-
+        
         if(err != EINTR)
         {
             /*
@@ -430,16 +386,12 @@ server_mt_process_udp(zdb *database, synced_thread_t *st)
             }
         }
 #else
-
-        st->udp_iovec.iov_len = sizeof(st->udp_mesg->buffer);
-        st->udp_msghdr.msg_controllen = ANCILIARY_BUFFER_SIZE;
-
-        n = recvmsg(fd, &st->udp_msghdr, 0);
+        n = recvmsg(st->fdsock, &st->udp_msghdr, 0);
         
         if(n >= 0)
         {
 #ifdef DEBUG
-            log_debug("server_mt_process_udp: recvmsg: got %d bytes", n);
+            log_debug("server_mt_process_udp: recvmsg: %{sockaddr}: %d bytes", st->udp_msghdr.msg_name, n);
 #endif
             break;
         }
@@ -456,12 +408,21 @@ server_mt_process_udp(zdb *database, synced_thread_t *st)
             if(err != EAGAIN)
             {
 #ifdef DEBUG
-                log_err("server_mt_process_udp: recvmsg error: %r", MAKE_ERRNO_ERROR(err));
+                log_err("server_mt_process_udp: recvmsg: %{sockaddr}: error: %r", st->udp_msghdr.msg_name, MAKE_ERRNO_ERROR(err));
 #endif
                 return;
             }
         }
 #endif
+        
+        if(dnscore_shuttingdown())
+        {
+            // shutdown in progress
+#ifdef DEBUG
+            log_debug("server_mt_process_udp: shutdown in progress");
+#endif
+            return;
+        }
     }
 
     mesg->received = n;
@@ -490,7 +451,7 @@ server_mt_process_udp(zdb *database, synced_thread_t *st)
                     {
                         local_statistics->udp_queries_count++;
 
-                        log_query(fd, mesg);
+                        log_query(st->fdsock, mesg);
 
                         switch(mesg->qtype)
                         {
@@ -530,7 +491,7 @@ server_mt_process_udp(zdb *database, synced_thread_t *st)
                             }
                             case TYPE_IXFR:
                             {
-                                MESSAGE_FLAGS_OR(mesg->buffer, QR_BITS|TC_BITS, 0); /** @todo IXFR UDP */
+                                MESSAGE_FLAGS_OR(mesg->buffer, QR_BITS|TC_BITS, 0); /** @todo 20120619 edf -- IXFR UDP */
                                 SET_U32_AT(mesg->buffer[4], 0);
                                 SET_U32_AT(mesg->buffer[8], 0);
                                 mesg->send_length = DNS_HEADER_LENGTH;
@@ -578,6 +539,8 @@ server_mt_process_udp(zdb *database, synced_thread_t *st)
             } // if message process succeeded
             else // an error occurred : no query to be done at all
             {
+                return_code = message_process_query(mesg);
+                
                 log_warn("query (%04hx) [%02x|%02x] error %i (%r) (%{sockaddrip})",
                          ntohs(MESSAGE_ID(mesg->buffer)),
                          MESSAGE_HIFLAGS(mesg->buffer),MESSAGE_LOFLAGS(mesg->buffer),
@@ -587,10 +550,12 @@ server_mt_process_udp(zdb *database, synced_thread_t *st)
 
                 local_statistics->udp_fp[mesg->status]++;
                 
-                if(return_code == UNPROCESSABLE_MESSAGE)
+#ifdef DEBUG
+                if(return_code == UNPROCESSABLE_MESSAGE && (g_config->server_flags & SERVER_FL_LOG_UNPROCESSABLE))
                 {
-                    log_memdump_ex(MODULE_MSG_HANDLE, MSG_ERR, mesg->buffer, mesg->received, 16, OSPRINT_DUMP_ALL);
+                    log_memdump_ex(MODULE_MSG_HANDLE, MSG_DEBUG, mesg->buffer, mesg->received, 16, OSPRINT_DUMP_ALL);
                 }
+#endif
                 
                 /*
                  * If not FE, or if we answer FE
@@ -633,7 +598,7 @@ server_mt_process_udp(zdb *database, synced_thread_t *st)
 
                         bool answer = MESSAGE_QR(mesg->buffer);
                         
-                        return_value = notify_process(database, mesg); // thread-safe
+                        return_value = notify_process(mesg); // thread-safe
                         
                         local_statistics->udp_fp[mesg->status]++;
                         
@@ -728,7 +693,7 @@ server_mt_process_udp(zdb *database, synced_thread_t *st)
 
                         local_statistics->udp_updates_count++;
 
-                        server_mt_process_udp_update(database, st);
+                        server_mt_process_udp_update(database, mesg);
                         
                         return; // NOT break;
 #else
@@ -806,7 +771,7 @@ server_mt_process_udp(zdb *database, synced_thread_t *st)
         }
     } // switch operation code
     
-    /** @todo still needs to verify RCODE */
+    /** @todo 20120619 edf -- still needs to verify RCODE */
 
 #ifdef DEBUG
     if(mesg->send_length < 12)
@@ -825,10 +790,12 @@ server_mt_process_udp(zdb *database, synced_thread_t *st)
     
 #ifdef DEBUG
     log_debug("server_mt_process_udp: sendto(%d, %p, %d, %d, %{sockaddr}, %d)", mesg->sockfd, mesg->buffer, mesg->send_length, 0, (struct sockaddr*)&mesg->other.sa, mesg->addr_len);
+#if DUMP_UDP_MT_OUTPUT_WIRE
     log_memdump_ex(g_server_logger, MSG_DEBUG5, mesg->buffer, mesg->send_length, 16, OSPRINT_DUMP_HEXTEXT);
+#endif
 #endif
     
-    while((sent = sendto(mesg->sockfd, mesg->buffer, mesg->send_length, 0, (struct sockaddr*)&mesg->other.sa, mesg->addr_len)) < 0)
+    while((sent = sendto(st->fdsock, mesg->buffer, mesg->send_length, 0, (struct sockaddr*)&mesg->other.sa, mesg->addr_len)) < 0)
     {
         int error_code = errno;
 
@@ -848,7 +815,7 @@ server_mt_process_udp(zdb *database, synced_thread_t *st)
     log_debug("sendmsg(%d, %p, %d", mesg->sockfd, &st->udp_msghdr, 0);
 #endif
     
-    while( (sent = sendmsg(mesg->sockfd, &st->udp_msghdr, 0)) < 0)
+    while( (sent = sendmsg(st->fdsock, &st->udp_msghdr, 0)) < 0)
     {
         int error_code = errno;
 
@@ -917,13 +884,13 @@ server_mt_query_loop_udp(void* parm)
     /*    ------------------------------------------------------------    */
 
     /* Clear and initialize mesg */
-    ZEROMEMORY(st->udp_mesg, sizeof (message_data));
+    ZEROMEMORY(st->udp_mesg, sizeof(message_data));
     
     st->udp_mesg->addr_len      = sizeof(st->udp_mesg->other);
     st->udp_mesg->protocol      = IPPROTO_UDP;
     st->udp_mesg->size_limit    = UDPPACKET_MAX_LENGTH;
-    st->udp_mesg->process_flags = ~0; /** @todo FIX ME */
-    st->udp_mesg->sockfd = st->intf->udp.sockfd;
+    st->udp_mesg->process_flags = ~0; /** @todo 20160106 edf -- FIX ME */    
+    st->udp_mesg->sockfd = st->fdsock;
     
     tcp_set_recvtimeout(st->udp_mesg->sockfd, 1, 0);
 
@@ -942,14 +909,21 @@ server_mt_query_loop_udp(void* parm)
     st->udp_msghdr.msg_namelen = st->udp_mesg->addr_len;
     st->udp_msghdr.msg_iov = &st->udp_iovec;
     st->udp_msghdr.msg_iovlen = 1;
-    MALLOC_OR_DIE(struct msghdr*, st->udp_msghdr.msg_control, ANCILIARY_BUFFER_SIZE, MSGHDR_TAG);
-    st->udp_msghdr.msg_controllen = ANCILIARY_BUFFER_SIZE;
 
+    if(ANCILIARY_BUFFER_SIZE > 0)
+    {
+        MALLOC_OR_DIE(struct msghdr*, st->udp_msghdr.msg_control, ANCILIARY_BUFFER_SIZE, MSGHDR_TAG);
+    }
+    else
+    {
+        st->udp_msghdr.msg_control = NULL;
+    }    
+    st->udp_msghdr.msg_controllen = ANCILIARY_BUFFER_SIZE;
     st->udp_msghdr.msg_flags = 0;
 
 #endif
     
-    log_debug("server-mt: reading on %p", st->intf);
+    log_debug("server-mt: reading on %i", st->fdsock);
     
     while(program_mode != SA_SHUTDOWN)
     {
@@ -958,7 +932,7 @@ server_mt_query_loop_udp(void* parm)
         server_mt_process_udp(g_config->database, st);
     }
     
-    log_debug("server-mt: stop reading on %p", st->intf); 
+    log_debug("server-mt: stop reading on %i", st->fdsock); 
     
     synced_set_terminated(st);
     
@@ -969,7 +943,6 @@ ya_result
 server_mt_query_loop()
 {
     ya_result return_code;
-    interface *intf;
 
     if(g_config->total_interfaces == 0)
     {
@@ -1024,65 +997,97 @@ server_mt_query_loop()
     
     if(reader_by_fd > cpu_count)
     {
-        log_warn("server-mt: using too many threads per address is counter-productive on highly loaded systems (%d > %d)", reader_by_fd, cpu_count);
+        log_warn("server-mt: using too many threads per address is counter-productive on highly loaded systems (%d >= %d)", reader_by_fd, cpu_count);
     }
     
     /*
      * 
      */
             
-    u32 itf_count = g_config->interfaces_limit - &g_config->interfaces[0];
+    int intf_count = server_context.listen_count;
     
     // ensure the number of udp thread by interface does not goes "too much" beyond a limit
     // recompute reader_by_fd if it does
     
-    if(reader_by_fd * itf_count <= SERVER_MAX_UDP_THREADS)
+    if(reader_by_fd * intf_count <= SERVER_MAX_UDP_THREADS)
     {
         reader_by_fd = MAX(reader_by_fd, 1);
     }
     else
     {
-        reader_by_fd = MAX(SERVER_MAX_UDP_THREADS / itf_count, 1);
+        reader_by_fd = MAX(SERVER_MAX_UDP_THREADS / intf_count, 1);
     }
     
-    synced_init(itf_count * reader_by_fd);
+    synced_init(intf_count * reader_by_fd);
     
     u32 tidx = 0;
+
+    int sockfd_idx = 0;
     
-    struct thread_pool_s *server_udp_thread_pool = thread_pool_init_ex(itf_count * reader_by_fd, 1, "server-udp-tp");
+    extern server_context_s server_context;
     
-    for(intf = g_config->interfaces; intf < g_config->interfaces_limit; intf++)
+    struct thread_pool_s *server_udp_thread_pool = thread_pool_init_ex(intf_count * reader_by_fd, 1, "server-udp-tp");
+    
+    for(int intf_idx = 0; intf_idx < intf_count; ++intf_idx)
     {
-        for(u32 r = 0; r < reader_by_fd; r++)
+        if(server_context.reuse)
         {
-            synced_threads.threads[tidx].intf = intf;
-            
-            log_debug("server_mt_query_loop: pooling #%d=%d fd=%d", tidx, synced_threads.threads[tidx].idx, intf->udp.sockfd);
-            
-            if(FAIL(return_code = thread_pool_enqueue_call(server_udp_thread_pool, server_mt_query_loop_udp, &synced_threads.threads[tidx], NULL, "server-mt-task")))
+            for(u32 r = 0; r < reader_by_fd; r++)
             {
-                log_err("unable to schedule task : %r", return_code);
+                synced_threads.threads[tidx].fdsock = server_context.udp_socket[sockfd_idx++];
                 
-                return return_code;
-            }
-            
-            tidx++;
-        }
-
-        maxfd = MAX(maxfd, intf->tcp.sockfd);
-
-        /*
-         * Update the select read set for the current interface (udp + tcp)
-         */
-
 #if UDP_USE_MESSAGES
-        int sockopt_dstaddr = 1;
-        setsockopt(intf->udp.sockfd, IPPROTO_IP, DSTADDR_SOCKOPT, &sockopt_dstaddr, sizeof(sockopt_dstaddr));
+                const int sockopt_dstaddr = 1;
+                setsockopt(synced_threads.threads[tidx].fdsock , IPPROTO_IP, DSTADDR_SOCKOPT, &sockopt_dstaddr, sizeof(sockopt_dstaddr));
 #endif
+                
+                synced_threads.threads[tidx].fdsock = synced_threads.threads[tidx].fdsock;
+            
+                log_info("thread #%i of UDP interface: %{hostaddr} using socket %i", r, server_context.listen[intf_idx], synced_threads.threads[tidx].fdsock);
 
-        FD_SET(intf->tcp.sockfd, &read_set_init);
+                if(FAIL(return_code = thread_pool_enqueue_call(server_udp_thread_pool, server_mt_query_loop_udp, &synced_threads.threads[tidx], NULL, "server-mt-task")))
+                {
+                    log_err("unable to start task : %r", return_code);
+
+                    return return_code;
+                }
+
+                tidx++;
+            }
+        }
+        else
+        {
+            int sockfd;
+
+            sockfd = server_context.udp_socket[sockfd_idx++];
+ 
+            maxfd = MAX(maxfd, sockfd);
+
+            for(u32 r = 0; r < reader_by_fd; r++)
+            {
+                synced_threads.threads[tidx].fdsock = sockfd;
+
+                log_info("thread #%i of UDP interface: %{hostaddr} using socket %i", r, server_context.listen[intf_idx], synced_threads.threads[tidx].fdsock);
+
+                if(FAIL(return_code = thread_pool_enqueue_call(server_udp_thread_pool, server_mt_query_loop_udp, &synced_threads.threads[tidx], NULL, "server-mt-task")))
+                {
+                    log_err("unable to start task : %r", return_code);
+
+                    return return_code;
+                }
+
+                tidx++;
+            }
+        }
     }
-    
+
+    for(int i = 0; i < server_context.tcp_socket_count; ++i)
+    {
+        int sockfd = server_context.tcp_socket[i];
+        maxfd = MAX(maxfd, sockfd);
+        FD_SET(sockfd, &read_set_init);
+    }
+
     maxfd++; /* pselect actually requires maxfd + 1 */
     
     /* compute maxfd plus one once and for all : done */
@@ -1102,9 +1107,8 @@ server_mt_query_loop()
         /* At this moment waits only for READ SET or timeout of x seconds */
 
         /*
-         * @note (p)select has known bugs on Linux & glibc
+         * @note (p)select has known bugs on Linux & glibc (man select)
          *
-         * @todo See man select about said bugs
          */
 
         return_code = pselect(maxfd,
@@ -1116,8 +1120,6 @@ server_mt_query_loop()
 
         if(return_code > 0) /* Are any bit sets by pselect ? */
         {
-            int sockfd;
-
             /* If pselect check for the correct sock file descriptor,
              * at this moment only READ SET
              */
@@ -1131,15 +1133,15 @@ server_mt_query_loop()
              *
              * Since the call is only local it should not have side effects.
              */
-
-            for(intf = g_config->interfaces; intf < g_config->interfaces_limit; intf++)
+            
+            for(int i = 0; i < server_context.tcp_socket_count; ++i)
             {
-                sockfd = intf->tcp.sockfd;
-
+                int sockfd = server_context.tcp_socket[i];
+                
                 if(FD_ISSET(sockfd, &read_set))
                 {
                     /* Jumps to the correct processing function */
-                    server_process_tcp(g_config->database, &intf->tcp);
+                    server_process_tcp(g_config->database, sockfd);
 
                     /* DEBUG static counts */
 
@@ -1215,7 +1217,9 @@ server_mt_query_loop()
                         /* server_statistics_sum.input_timeout_count += stats->input_timeout_count; */
                         
                         server_statistics_sum.udp_output_size_total += stats->udp_output_size_total;
+#if SHOW_REFERRAL
                         server_statistics_sum.udp_referrals_count += stats->udp_referrals_count;
+#endif
                         server_statistics_sum.udp_input_count += stats->udp_input_count;
                         server_statistics_sum.udp_dropped_count += stats->udp_dropped_count;
                         server_statistics_sum.udp_queries_count += stats->udp_queries_count;
@@ -1244,7 +1248,7 @@ server_mt_query_loop()
                     zalloc_print_stats(termout);
 #endif
 #if DNSCORE_HAS_MALLOC_DEBUG_SUPPORT
-                    debug_stat(FALSE); // do NOT enable the dump
+                    debug_stat(DEBUG_STAT_SIZES|DEBUG_STAT_TAGS); // do NOT enable the dump
 #endif
                     journal_log_status();
                     
@@ -1272,21 +1276,22 @@ server_mt_query_loop()
     thread_pool_destroy(server_udp_thread_pool);
     server_udp_thread_pool = NULL;
 
-    /**
-     * @todo READ THIS
-     *
-     * At this point, there one or more background readers could still be working.
-     * Destroying the database at this time (they do not expect a writer) will crash them.
-     * So either we don't destroy the database (required to ensure there has been no corruption nor leaks)
-     * Either we wait for them to finish (but signing 3M records takes time)
-     * Either ...
-     *
-     * BTW: the odd "non-reproducible" crash I've got on Yadifa is this ending race happening.
-     *
-     */
-
     log_debug("shutting down (pid = %u)", getpid());
     
     return SUCCESS;
 }
+
+ya_result
+server_mt_context_init(int workers_per_interface)
+{
+    server_context.thread_per_udp_worker_count = 1; // set in stone
+    server_context.thread_per_tcp_worker_count = 1; // set in stone
+    server_context.udp_unit_per_interface = MAX(workers_per_interface, 1);;
+    server_context.tcp_unit_per_interface = 1;    
+    server_context.reuse = 0;
+    server_context.ready = 1;
+    return SUCCESS;
+}
+
+
 /** @} */
